@@ -26,25 +26,30 @@
 #ifndef GUARD_MIOPEN_CONV_BN_ACTIV_INFER_DRIVER_HPP
 #define GUARD_MIOPEN_CONV_BN_ACTIV_INFER_DRIVER_HPP
 
-#include "../test/verify.hpp"
 #include "InputFlags.hpp"
 #include "driver.hpp"
 #include "miopen_ConvBatchNormActivHost.hpp"
+#include "mloNeuronHost.hpp"
+#include "random.hpp"
 #include "tensor_driver.hpp"
 #include "timer.hpp"
+#include "util_driver.hpp"
+
+#include "../test/verify.hpp"
+
+#include <miopen/env.hpp>
+#include <miopen/handle.hpp>
+#include <miopen/miopen.h>
+#include <miopen/tensor.hpp>
+
 #include <algorithm>
+#include <cassert>
 #include <cmath>
 #include <cstdlib>
 #include <float.h>
 #include <memory>
-#include <miopen/miopen.h>
-#include <miopen/handle.hpp>
-#include <miopen/tensor.hpp>
 #include <numeric>
 #include <vector>
-#include <cassert>
-#include "random.hpp"
-#include "mloNeuronHost.hpp"
 
 #define MIO_BN_DEBUG 0
 #define MIO_BN_MAX_DEBUGLOOP 65536
@@ -58,14 +63,10 @@
 #define RMSTOL_FP32 1e-4
 #define RMSTOL_FP16 0.5e-3
 
-#ifdef MIOPEN_BACKEND_HIP
-#ifndef CL_SUCCESS
-#define CL_SUCCESS 0
-#endif
-#endif
-
 #define CBA_DEBUG_VALUES 0
 
+MIOPEN_DECLARE_ENV_VAR_BOOL(MIOPEN_DRIVER_PAD_BUFFERS_2M)
+
 //"Fusion mode (cbna = 0, cna = 1, na = 2, cn = 3, cba = 4, ca = 5, cb = 6) (Default=cbna)",
 typedef enum
 {
@@ -512,15 +513,13 @@ template <typename Tgpu, typename Tref>
 int CBAInferFusionDriver<Tgpu, Tref>::createSaveBuffers()
 {
 
+    status_t status = STATUS_SUCCESS;
 #if MIOPEN_BACKEND_OPENCL
-    cl_int status = CL_SUCCESS;
     cl_context ctx;
     clGetCommandQueueInfo(q, CL_QUEUE_CONTEXT, sizeof(cl_context), &ctx, nullptr);
-#elif MIOPEN_BACKEND_HIP
-    int status   = 0;
 #endif
 
-    if(status != CL_SUCCESS)
+    if(status != STATUS_SUCCESS)
         printf("Error copying data to GPU\n");
 
     return miopenStatusSuccess;
@@ -530,13 +529,10 @@ template <typename Tgpu, typename Tref>
 int CBAInferFusionDriver<Tgpu, Tref>::createRunningBuffers()
 {
 
+    status_t status = STATUS_SUCCESS;
+    DEFINE_CONTEXT(ctx);
 #if MIOPEN_BACKEND_OPENCL
-    cl_int status = CL_SUCCESS;
-    cl_context ctx;
     clGetCommandQueueInfo(q, CL_QUEUE_CONTEXT, sizeof(cl_context), &ctx, nullptr);
-#elif MIOPEN_BACKEND_HIP
-    int status   = 0;
-    uint32_t ctx = 0;
 #endif
 
     if(useBatchNorm)
@@ -566,7 +562,7 @@ int CBAInferFusionDriver<Tgpu, Tref>::createRunningBuffers()
         // GPU data transfer
         status |= runningMean_dev->ToGPU(q, runningMean.data());
         status |= runningVariance_dev->ToGPU(q, runningVariance.data());
-        if(status != CL_SUCCESS)
+        if(status != STATUS_SUCCESS)
         {
             printf("Error copying data to GPU\n");
             exit(EXIT_FAILURE); // NOLINT (concurrency-mt-unsafe)
@@ -584,13 +580,10 @@ template <typename Tgpu, typename Tref>
 int CBAInferFusionDriver<Tgpu, Tref>::AllocateBuffersAndCopy()
 {
 
+    status_t status = STATUS_SUCCESS;
+    DEFINE_CONTEXT(ctx);
 #if MIOPEN_BACKEND_OPENCL
-    cl_int status = CL_SUCCESS;
-    cl_context ctx;
     clGetCommandQueueInfo(q, CL_QUEUE_CONTEXT, sizeof(cl_context), &ctx, nullptr);
-#elif MIOPEN_BACKEND_HIP
-    int status   = 0;
-    uint32_t ctx = 0;
 #endif
 
     size_t in_sz  = GetTensorSize(inputTensor);
@@ -608,7 +601,7 @@ int CBAInferFusionDriver<Tgpu, Tref>::AllocateBuffersAndCopy()
     else
         out_sz = in_sz; // This is for N+A so the output is the same as the input size
 
-    if(miopen::IsEnabled(ENV(MIOPEN_DRIVER_PAD_BUFFERS_2M)))
+    if(env::enabled(MIOPEN_DRIVER_PAD_BUFFERS_2M))
     {
         PadBufferSize(wei_sz, sizeof(Tgpu));
     }
@@ -648,8 +641,8 @@ int CBAInferFusionDriver<Tgpu, Tref>::AllocateBuffersAndCopy()
             scale[i] = 1.; // prng::gen_canonical<Tgpu>(); // 1.0;
             bias[i]  = 10.;
 #else
-            scale[i] = prng::gen_canonical<Tgpu>();
-            bias[i]  = prng::gen_canonical<Tgpu>();
+            scale[i]           = prng::gen_canonical<Tgpu>();
+            bias[i]            = prng::gen_canonical<Tgpu>();
 #endif
         }
         status |= scale_dev->ToGPU(q, scale.data());
@@ -674,9 +667,9 @@ int CBAInferFusionDriver<Tgpu, Tref>::AllocateBuffersAndCopy()
         in_host[i] = static_cast<double>(rval);
         in[i]      = rval;
 #else
-        auto rval  = prng::gen_canonical<Tgpu>();
+        auto rval = prng::gen_canonical<Tgpu>();
         in_host[i] = static_cast<double>(rval);
-        in[i]      = rval;
+        in[i] = rval;
 #endif
     }
 
@@ -697,7 +690,7 @@ int CBAInferFusionDriver<Tgpu, Tref>::AllocateBuffersAndCopy()
     status |= in_dev->ToGPU(q, in.data());
     status |= createRunningBuffers();
 
-    if(status != CL_SUCCESS)
+    if(status != STATUS_SUCCESS)
         printf("Fatal: Error copying data to GPU\nExiting...\n\n");
 
     return miopenStatusSuccess;
