@@ -26,31 +26,32 @@
 #ifndef GUARD_MIOPEN_REDUCE_DRIVER_HPP
 #define GUARD_MIOPEN_REDUCE_DRIVER_HPP
 
-#include "../test/verify.hpp"
 #include "InputFlags.hpp"
 #include "driver.hpp"
+#include "miopen_Reduction.hpp"
+#include "random.hpp"
 #include "tensor_driver.hpp"
 #include "timer.hpp"
-#include <algorithm>
-#include <cstdlib>
-#include <float.h>
-#include <memory>
+#include "util_driver.hpp"
+#include "util_file.hpp"
+
+#include "../test/verify.hpp"
+
 #include <miopen/miopen.h>
 #include <miopen/reduce_common.hpp>
 #include <miopen/tensor.hpp>
-#include <numeric>
-#include <vector>
-#include <string>
-#include <cassert>
-#include <type_traits>
-#if !defined(_WIN32)
-#include <half/half.hpp>
-#else
-#include <half.hpp>
-#endif
-#include "random.hpp"
 
-#include "miopen_Reduction.hpp"
+#include <half/half.hpp>
+
+#include <algorithm>
+#include <cassert>
+#include <cstdlib>
+#include <float.h>
+#include <memory>
+#include <numeric>
+#include <string>
+#include <type_traits>
+#include <vector>
 
 template <typename Tgpu, typename Tref>
 class ReduceDriver : public Driver
@@ -321,12 +322,9 @@ int ReduceDriver<Tgpu, Tref>::AllocateBuffersAndCopy()
                                                  : this->ws_sizeInBytes / (sizeof(Tgpu) + sizeof(int));
     size_t indices_nelem = this->indices_sizeInBytes / sizeof(int);
 
+    DEFINE_CONTEXT(ctx);
 #if MIOPEN_BACKEND_OPENCL
-    cl_context ctx;
-
     clGetCommandQueueInfo(q, CL_QUEUE_CONTEXT, sizeof(cl_context), &ctx, nullptr);
-#elif MIOPEN_BACKEND_HIP
-    uint32_t ctx = 0;
 #endif
     in_dev  = std::unique_ptr<GPUMem>(new GPUMem(ctx, in_nelem, sizeof(Tgpu)));
     out_dev = std::unique_ptr<GPUMem>(new GPUMem(ctx, out_nelem, sizeof(Tgpu)));
@@ -356,15 +354,11 @@ int ReduceDriver<Tgpu, Tref>::AllocateBuffersAndCopy()
         };
     };
 
-#if MIOPEN_BACKEND_OPENCL
-    cl_int status;
-#elif MIOPEN_BACKEND_HIP
-    int status;
-#endif
+    status_t status;
     status = in_dev->ToGPU(q, in.data());
     status |= out_dev->ToGPU(q, out.data());
 
-    if(status != CL_SUCCESS)
+    if(status != STATUS_SUCCESS)
         printf("Error copying data to GPU\n");
 
     return miopenStatusSuccess;
