@@ -27,29 +27,34 @@
 #define GUARD_MIOPEN_RNN_DRIVER_HPP
 
 #include "InputFlags.hpp"
-#include "rnn_verify_gemm.hpp"
-#include "lstm_verify_gemm.hpp"
-#include "gru_verify_gemm.hpp"
 #include "driver.hpp"
+#include "gru_verify_gemm.hpp"
+#include "lstm_verify_gemm.hpp"
+#include "mloConvHost.hpp"
+#include "random.hpp"
+#include "rnn_verify_gemm.hpp"
 #include "tensor_driver.hpp"
 #include "timer.hpp"
 #include "util_driver.hpp"
-#include "random.hpp"
+#include "util_file.hpp"
+
 #include <../test/verify.hpp>
-#include <algorithm>
-#include <cstdlib>
-#include <cstring>
-#include <cfloat>
-#include <fstream>
-#include <memory>
+
+#include <miopen/env.hpp>
 #include <miopen/miopen.h>
 #include <miopen/rnn.hpp>
 #include <miopen/tensor.hpp>
-#include <miopen/env.hpp>
+
+#include <algorithm>
+#include <array>
+#include <cfloat>
+#include <cstdlib>
+#include <cstring>
+#include <fstream>
+#include <memory>
 #include <numeric>
 #include <sstream>
 #include <vector>
-#include <array>
 
 template <typename Tgpu, typename Tref>
 class RNNDriver : public Driver
@@ -153,7 +158,7 @@ private:
     std::vector<Tref> cy_host;
     std::vector<Tref> dhx_host;
     std::vector<Tref> dcx_host;
-    std::vector<prngStates> dropout_states_host;
+    std::vector<rocrand_state_xorwow> dropout_states_host;
 
     miopenRNNDescriptor_t rnnDesc;
 
@@ -190,10 +195,6 @@ int RNNDriver<Tgpu, Tref>::ParseCmdLineArgs(int argc, char* argv[])
 {
     inflags.Parse(argc, argv);
 
-    if(inflags.GetValueInt("time") == 1)
-    {
-        miopenEnableProfiling(GetHandle(), true);
-    }
     return miopenStatusSuccess;
 }
 
@@ -254,9 +255,16 @@ int RNNDriver<Tgpu, Tref>::AddCmdLineArgs()
                          "Use specified directory to cache verification data. Off by default.",
                          "string");
     */
-    inflags.AddInputFlag("time", 't', "0", "Time Each Layer (Default=0)", "int");
     inflags.AddInputFlag(
-        "wall", 'w', "0", "Wall-clock Time Each Layer, Requires time == 1 (Default=0)", "int");
+        "wall",
+        'w',
+        "0",
+        "Wall-clock, for host and gpu, Time Each Layer,       Disabled                = 0,\
+        OldWallClock            = 1,\
+        SeparateClocksSynced    = 2,\
+        SeparateClocksNotSynced = 3 \
+        (Default = 0) ",
+        "int");
     inflags.AddInputFlag("dump_output", 'o', "0", "Dumps the output buffers (Default=0)", "int");
     /*  // DL: These have not been implemented. Removing them for now.
         inflags.AddInputFlag("in_data", 'd', "", "Input data filename (Default=)", "string");
@@ -479,18 +487,15 @@ int RNNDriver<Tgpu, Tref>::SetRNNDescriptorFromCmdLineArgs()
 
         size_t statesSizeInBytes = 0;
         miopenDropoutGetStatesSize(GetHandle(), &statesSizeInBytes);
-        size_t states_size = statesSizeInBytes / sizeof(prngStates);
+        size_t states_size = statesSizeInBytes / sizeof(rocrand_state_xorwow);
 
+        DEFINE_CONTEXT(ctx);
 #if MIOPEN_BACKEND_OPENCL
-        cl_context ctx;
-
         clGetCommandQueueInfo(q, CL_QUEUE_CONTEXT, sizeof(cl_context), &ctx, nullptr);
-#elif MIOPEN_BACKEND_HIP
-        uint32_t ctx = 0;
 #endif
 
         dropout_states_dev =
-            std::unique_ptr<GPUMem>(new GPUMem(ctx, states_size, sizeof(prngStates)));
+            std::unique_ptr<GPUMem>(new GPUMem(ctx, states_size, sizeof(rocrand_state_xorwow)));
 
         miopenSetDropoutDescriptor(DropoutDesc,
                                    GetHandle(),
@@ -564,12 +569,9 @@ int RNNDriver<Tgpu, Tref>::AllocateBuffersAndCopy()
     workSpace_sz /= sizeof(Tgpu);
     reserveSpace_sz = (reserveSpace_sz + sizeof(Tgpu) - 1) / sizeof(Tgpu);
 
+    DEFINE_CONTEXT(ctx);
 #if MIOPEN_BACKEND_OPENCL
-    cl_context ctx;
-
     clGetCommandQueueInfo(q, CL_QUEUE_CONTEXT, sizeof(cl_context), &ctx, nullptr);
-#elif MIOPEN_BACKEND_HIP
-    uint32_t ctx = 0;
 #endif
 
     in_dev           = std::unique_ptr<GPUMem>(new GPUMem(ctx, in_sz, sizeof(Tgpu)));
@@ -619,7 +621,7 @@ int RNNDriver<Tgpu, Tref>::AllocateBuffersAndCopy()
     int nseq              = inflags.GetValueInt("seq_len");
     std::vector<int> in_n = GetInputTensorLengthsFromCmdLine();
     std::size_t inputBatchLenSum;
-    inputBatchLenSum = std::accumulate(in_n.begin(), in_n.begin() + nseq, 0);
+    inputBatchLenSum = std::accumulate(in_n.begin(), in_n.begin() + nseq, 0ULL);
 
     int hid_h = inflags.GetValueInt("hid_h");
     int layer = inflags.GetValueInt("num_layer");
@@ -733,13 +735,7 @@ int RNNDriver<Tgpu, Tref>::AllocateBuffersAndCopy()
         dumpBufferToFile("dump_wei.bin", wei.data(), wei_sz);
     }
 
-#if MIOPEN_BACKEND_OPENCL
-    cl_int status;
-#elif MIOPEN_BACKEND_HIP
-#define CL_SUCCESS 0
-    int status;
-#endif
-
+    status_t status;
     status = in_dev->ToGPU(q, in.data());
     status |= wei_dev->ToGPU(q, wei.data());
     status |= out_dev->ToGPU(q, out.data());
@@ -748,7 +744,7 @@ int RNNDriver<Tgpu, Tref>::AllocateBuffersAndCopy()
     status |= workspace_dev->ToGPU(q, workspace.data());
     status |= reservespace_dev->ToGPU(q, reservespace.data());
 
-    if(status != CL_SUCCESS)
+    if(status != STATUS_SUCCESS)
         printf("Error copying data to GPU\n");
 
     if(inflags.GetValueInt("forw") != 2)
@@ -756,7 +752,7 @@ int RNNDriver<Tgpu, Tref>::AllocateBuffersAndCopy()
         status = hy_dev->ToGPU(q, hy.data());
         status |= cy_dev->ToGPU(q, cy.data());
 
-        if(status != CL_SUCCESS)
+        if(status != STATUS_SUCCESS)
             printf("Error copying data to GPU\n");
     }
 
@@ -770,7 +766,7 @@ int RNNDriver<Tgpu, Tref>::AllocateBuffersAndCopy()
         status |= dhy_dev->ToGPU(q, dhy.data());
         status |= dcy_dev->ToGPU(q, dcy.data());
 
-        if(status != CL_SUCCESS)
+        if(status != STATUS_SUCCESS)
             printf("Error copying data to GPU\n");
     }
 
@@ -791,9 +787,7 @@ int RNNDriver<Tgpu, Tref>::RunForwardGPU()
     if(inflags.GetValueInt("forw") != 0 && !(inflags.GetValueInt("forw") & 1))
         return miopenStatusSuccess;
 
-    Timer t;
-    float wl_time_forward = 0.0;
-    float kl_time_forward = 0.0;
+    RNNCombTimeLoger t(GetStream(), inflags.GetValueInt("iter"), inflags.GetValueInt("wall"));
 
     for(int i = 0; i < inflags.GetValueInt("iter"); i++)
     {
@@ -808,8 +802,7 @@ int RNNDriver<Tgpu, Tref>::RunForwardGPU()
             workspace_dev->ToGPU(q, workspace.data());
             reservespace_dev->ToGPU(q, reservespace.data());
         }
-
-        START_TIME
+        t.Start();
         if(inflags.GetValueInt("fwdtype") == 0)
         {
             miopenRNNForwardTraining(GetHandle(),
@@ -861,31 +854,14 @@ int RNNDriver<Tgpu, Tref>::RunForwardGPU()
                                       workspace_dev->GetMem(),
                                       workspace_dev->GetSize());
         }
-        miopen::deref(GetHandle()).Finish();
-        STOP_TIME
 
-        if(i > 0 || inflags.GetValueInt("iter") == 1)
-        {
-            float time = 0.0;
-            miopenGetKernelTime(GetHandle(), &time);
-            // printf("wall time: %f\n", t.gettime_ms());
-            wl_time_forward += t.gettime_ms();
-            kl_time_forward += time;
-        }
+        t.StopAndPush();
     }
-
-    if(inflags.GetValueInt("time") == 1)
-    {
-        int n_iter = inflags.GetValueInt("iter") > 1 ? inflags.GetValueInt("iter") - 1
-                                                     : inflags.GetValueInt("iter");
-        printf("GPU Kernel Time Forward RNN Elapsed: %f ms\n", kl_time_forward / n_iter);
-    }
-
+    miopen::deref(GetHandle()).Finish();
     if(WALL_CLOCK)
     {
-        int n_iter = inflags.GetValueInt("iter") > 1 ? inflags.GetValueInt("iter") - 1
-                                                     : inflags.GetValueInt("iter");
-        printf("Wall-clock Time Forward RNN Elapsed: %f ms\n", wl_time_forward / n_iter);
+        printf("Forward RNN time results:\n");
+        t.Print();
     }
 
     out_dev->FromGPU(GetStream(), out.data());
@@ -903,16 +879,6 @@ int RNNDriver<Tgpu, Tref>::RunForwardGPU()
     return miopenStatusSuccess;
 }
 
-std::tuple<size_t, size_t>
-GetTempPackedBuffersSize(std::vector<int> batchs, int in_vec, int out_vec)
-{
-    size_t total_batch = std::accumulate(batchs.begin(), batchs.end(), 0);
-
-    size_t in_buff_size  = total_batch * in_vec;
-    size_t out_buff_size = total_batch * out_vec;
-    return {in_buff_size, out_buff_size};
-}
-
 template <typename Tgpu>
 void ChangeDataPadding(std::vector<Tgpu>& src_array,
                        std::vector<Tgpu>& dst_array,
@@ -1128,15 +1094,12 @@ int RNNDriver<Tgpu, Tref>::RunBackwardGPU()
 
     if((inflags.GetValueInt("forw") & 2) || (inflags.GetValueInt("forw") == 0))
     {
-        Timer t;
-        float wl_time_backward_data = 0.0;
-        float kl_time_backward_data = 0.0;
-
+        RNNCombTimeLoger t(GetStream(), inflags.GetValueInt("iter"), inflags.GetValueInt("wall"));
         workspace_dev->ToGPU(q, workspace.data());
 
         for(int i = 0; i < inflags.GetValueInt("iter"); i++)
         {
-            START_TIME
+            t.Start();
             ret = miopenRNNBackwardData(GetHandle(),
                                         rnnDesc,
                                         adjustedSeqLen,
@@ -1164,31 +1127,13 @@ int RNNDriver<Tgpu, Tref>::RunBackwardGPU()
                                         workspace_dev->GetSize(),
                                         reservespace_dev->GetMem(),
                                         reservespace_dev->GetSize());
-            miopen::deref(GetHandle()).Finish();
-            STOP_TIME
-            if(i > 0 || inflags.GetValueInt("iter") == 1)
-            {
-                float time = 0.0;
-                miopenGetKernelTime(GetHandle(), &time);
-                wl_time_backward_data += t.gettime_ms();
-                kl_time_backward_data += time;
-            }
+            t.StopAndPush();
         }
-
-        if(inflags.GetValueInt("time") == 1)
-        {
-            int n_iter = inflags.GetValueInt("iter") > 1 ? inflags.GetValueInt("iter") - 1
-                                                         : inflags.GetValueInt("iter");
-            printf("GPU Kernel Time Backward Data RNN Elapsed: %f ms\n",
-                   kl_time_backward_data / n_iter);
-        }
-
+        miopen::deref(GetHandle()).Finish();
         if(WALL_CLOCK)
         {
-            int n_iter = inflags.GetValueInt("iter") > 1 ? inflags.GetValueInt("iter") - 1
-                                                         : inflags.GetValueInt("iter");
-            printf("Wall-clock Time Backward Data RNN Elapsed: %f ms\n",
-                   wl_time_backward_data / n_iter);
+            printf("Backward Data RNN time results:\n");
+            t.Print();
         }
 
         din_dev->FromGPU(GetStream(), din.data());
@@ -1199,13 +1144,11 @@ int RNNDriver<Tgpu, Tref>::RunBackwardGPU()
 
     if((inflags.GetValueInt("forw") & 4) || (inflags.GetValueInt("forw") == 0))
     {
-        Timer t;
-        float wl_time_backward_weight = 0.0;
-        float kl_time_backward_weight = 0.0;
+        RNNCombTimeLoger t(GetStream(), inflags.GetValueInt("iter"), inflags.GetValueInt("wall"));
 
         for(int i = 0; i < inflags.GetValueInt("iter"); i++)
         {
-            START_TIME
+            t.Start();
             ret = miopenRNNBackwardWeights(GetHandle(),
                                            rnnDesc,
                                            adjustedSeqLen,
@@ -1221,42 +1164,18 @@ int RNNDriver<Tgpu, Tref>::RunBackwardGPU()
                                            workspace_dev->GetSize(),
                                            reservespace_dev->GetMem(),
                                            reservespace_dev->GetSize());
-            miopen::deref(GetHandle()).Finish();
-            STOP_TIME
-            if(i > 0 || inflags.GetValueInt("iter") == 1)
-            {
-                float time = 0.0;
-                miopenGetKernelTime(GetHandle(), &time);
-                wl_time_backward_weight += t.gettime_ms();
-                kl_time_backward_weight += time;
-            }
-        }
-
-        if(inflags.GetValueInt("time") == 1)
-        {
-            int n_iter = inflags.GetValueInt("iter") > 1 ? inflags.GetValueInt("iter") - 1
-                                                         : inflags.GetValueInt("iter");
-            printf("GPU Kernel Time Backward Weights RNN Elapsed: %f ms\n",
-                   kl_time_backward_weight / n_iter);
+            t.StopAndPush();
         }
+        miopen::deref(GetHandle()).Finish();
 
         if(WALL_CLOCK)
         {
-            int n_iter = inflags.GetValueInt("iter") > 1 ? inflags.GetValueInt("iter") - 1
-                                                         : inflags.GetValueInt("iter");
-            printf("Wall-clock Time Backward Weights RNN Elapsed: %f ms\n",
-                   wl_time_backward_weight / n_iter);
+            printf("Backward Weights RNN time results:\n");
+            t.Print();
         }
 
         dwei_dev->FromGPU(GetStream(), dwei.data());
     }
-    /*
-       if(inflags.GetValueInt("dump_output"))
-       {
-       dumpBufferToFile("dump_bwd_din_gpu.bin", din.data(), din.size());
-       dumpBufferToFile("dump_bwd_dwei_gpu.bin", dwei.data(), dwei.size());
-       }
-       */
 
     return ret;
 }
