@@ -25,37 +25,38 @@
  *******************************************************************************/
 #pragma once
 
-#include <algorithm>
-#include <cstdlib>
-#include <cstring>
-#include <cfloat>
-#include <fstream>
-#include <memory>
-#include <initializer_list>
-#include <numeric>
-#include <sstream>
-#include <vector>
-#include <array>
-
-#include <miopen/errors.hpp>
-#include <miopen/logger.hpp>
-#include <miopen/tensor_ops.hpp>
-#include <miopen/miopen.h>
-#include <miopen/rnn.hpp>
-#include <miopen/tensor.hpp>
-#include <miopen/env.hpp>
-#include <../test/verify.hpp>
-//#include <../test/rnn_util.hpp>
-
 #include "InputFlags.hpp"
-#include "rnn_verify_gemm.hpp"
-#include "lstm_verify_gemm.hpp"
-#include "gru_verify_gemm.hpp"
 #include "driver.hpp"
+#include "gru_verify_gemm.hpp"
+#include "lstm_verify_gemm.hpp"
+#include "random.hpp"
+#include "rnn_verify_gemm.hpp"
 #include "tensor_driver.hpp"
 #include "timer.hpp"
 #include "util_driver.hpp"
-#include "random.hpp"
+#include "util_file.hpp"
+
+#include <../test/verify.hpp>
+
+#include <miopen/env.hpp>
+#include <miopen/errors.hpp>
+#include <miopen/logger.hpp>
+#include <miopen/miopen.h>
+#include <miopen/rnn.hpp>
+#include <miopen/tensor.hpp>
+#include <miopen/tensor_ops.hpp>
+
+#include <algorithm>
+#include <array>
+#include <cfloat>
+#include <cstdlib>
+#include <cstring>
+#include <fstream>
+#include <initializer_list>
+#include <memory>
+#include <numeric>
+#include <sstream>
+#include <vector>
 
 std::vector<size_t> get_default_time_strides(int vec_size, const std::vector<int>& seq_array)
 {
@@ -270,7 +271,7 @@ private:
     std::vector<Tref> cy_host;
     std::vector<Tref> dhx_host;
     std::vector<Tref> dcx_host;
-    std::vector<prngStates> dropout_states_host;
+    std::vector<rocrand_state_xorwow> dropout_states_host;
 
     miopenRNNDescriptor_t rnnDesc;
 
@@ -293,11 +294,6 @@ int RNNSeqDriver<Tgpu, Tref>::ParseCmdLineArgs(int argc, char* argv[])
 {
     inflags.Parse(argc, argv);
 
-    if(inflags.GetValueInt("time") == 1)
-    {
-        miopenEnableProfiling(GetHandle(), true);
-    }
-
     int nn_dir = inflags.GetValueInt("forw");
     if(inflags.GetValueInt("fwdtype") == 1 && !(nn_dir == 0 || nn_dir == 1))
     {
@@ -486,10 +482,17 @@ int RNNSeqDriver<Tgpu, Tref>::AddCmdLineArgs()
 
     inflags.AddInputFlag("iter", 'i', "1", "Number of Iterations (Default=1)", "int");
     inflags.AddInputFlag("verify", 'V', "1", "Verify Each Layer (Default=1)", "int");
-    inflags.AddInputFlag("time", 't', "0", "Time Each Layer (Default=0)", "int");
 
     inflags.AddInputFlag(
-        "wall", 'w', "0", "Wall-clock Time Each Layer, Requires time == 1 (Default=0)", "int");
+        "wall",
+        'w',
+        "0",
+        "Wall-clock, for host and gpu, Time Each Layer,       Disabled                = 0,\
+        OldWallClock            = 1,\
+        SeparateClocksSynced    = 2,\
+        SeparateClocksNotSynced = 3 \
+        (Default = 0) ",
+        "int");
     inflags.AddInputFlag("dump_output", 'o', "0", "Dumps the output buffers (Default=0)", "int");
     /*  // DL: These have not been implemented. Removing them for now.
         inflags.AddInputFlag("in_data", 'd', "", "Input data filename (Default=)", "string");
@@ -714,18 +717,15 @@ int RNNSeqDriver<Tgpu, Tref>::SetRNNDescriptorFromCmdLineArgs()
 
         size_t statesSizeInBytes = 0;
         miopenDropoutGetStatesSize(GetHandle(), &statesSizeInBytes);
-        size_t states_size = statesSizeInBytes / sizeof(prngStates);
+        size_t states_size = statesSizeInBytes / sizeof(rocrand_state_xorwow);
 
+        DEFINE_CONTEXT(ctx);
 #if MIOPEN_BACKEND_OPENCL
-        cl_context ctx;
-
         clGetCommandQueueInfo(q, CL_QUEUE_CONTEXT, sizeof(cl_context), &ctx, nullptr);
-#elif MIOPEN_BACKEND_HIP
-        uint32_t ctx = 0;
 #endif
 
         dropout_states_dev =
-            std::unique_ptr<GPUMem>(new GPUMem(ctx, states_size, sizeof(prngStates)));
+            std::unique_ptr<GPUMem>(new GPUMem(ctx, states_size, sizeof(rocrand_state_xorwow)));
 
         miopenSetDropoutDescriptor(DropoutDesc,
                                    GetHandle(),
@@ -768,7 +768,7 @@ inline size_t Get3DNoVECTensorSize(miopenTensorDescriptor_t& tensor)
     assert(miopen::deref(tensor).IsPacked() &&
            "GetTensorSize should not be used on an unpacked tensor.");
     const auto len = GetTensorLengths(tensor);
-    size_t sz      = std::accumulate(len.begin(), len.end(), 1, std::multiplies<size_t>());
+    size_t sz      = std::accumulate(len.begin(), len.end(), 1ULL, std::multiplies<size_t>());
     return sz;
 }
 
@@ -829,7 +829,7 @@ int RNNSeqDriver<Tgpu, Tref>::AllocateBuffersAndCopy()
     const std::vector<int> out_lens = GetOutputTensorLengthsFromCmdLine();
 
     const size_t vectors_cnt_host =
-        std::accumulate(sorted_seq_lens.begin(), sorted_seq_lens.end(), 0);
+        std::accumulate(sorted_seq_lens.begin(), sorted_seq_lens.end(), 0ULL);
     const size_t vectors_cnt_gpu =
         io_layout == miopenRNNDataSeqMajorNotPadded ? vectors_cnt_host : in_lens[0] * in_lens[1];
 
@@ -1075,9 +1075,7 @@ int RNNSeqDriver<Tgpu, Tref>::RunForwardGPU()
     if(inflags.GetValueInt("forw") != 0 && !(inflags.GetValueInt("forw") & 1))
         return miopenStatusSuccess;
 
-    Timer t;
-    float wl_time_forward = 0.0;
-    float kl_time_forward = 0.0;
+    RNNCombTimeLoger t(GetStream(), inflags.GetValueInt("iter"), inflags.GetValueInt("wall"));
 
     from_gpu_out = std::vector<Tgpu>(out_dev->GetSize() / sizeof(Tgpu), static_cast<Tgpu>(0));
 
@@ -1087,7 +1085,7 @@ int RNNSeqDriver<Tgpu, Tref>::RunForwardGPU()
         workspace_dev->ToGPU(q, workspace.data());
         reservespace_dev->ToGPU(q, reservespace.data());
 
-        START_TIME
+        t.Start();
         miopenRNNForward(GetHandle(),
                          rnnDesc,
                          fwd_type,
@@ -1108,32 +1106,14 @@ int RNNSeqDriver<Tgpu, Tref>::RunForwardGPU()
                          reservespace_dev->GetMem(),
                          reservespace_dev->GetSize());
 
-        miopen::deref(GetHandle()).Finish();
-        STOP_TIME
-
-        // TODO chage timing system
-        if(i > 0 || inflags.GetValueInt("iter") == 1)
-        {
-            float time = 0.0;
-            miopenGetKernelTime(GetHandle(), &time);
-            // printf("wall time: %f\n", t.gettime_ms());
-            wl_time_forward += t.gettime_ms();
-            kl_time_forward += time;
-        }
-    }
-
-    if(inflags.GetValueInt("time") == 1)
-    {
-        int n_iter = inflags.GetValueInt("iter") > 1 ? inflags.GetValueInt("iter") - 1
-                                                     : inflags.GetValueInt("iter");
-        printf("GPU Kernel Time Forward RNN Elapsed: %f ms\n", kl_time_forward / n_iter);
+        t.StopAndPush();
     }
 
+    miopen::deref(GetHandle()).Finish();
     if(WALL_CLOCK)
     {
-        int n_iter = inflags.GetValueInt("iter") > 1 ? inflags.GetValueInt("iter") - 1
-                                                     : inflags.GetValueInt("iter");
-        printf("Wall-clock Time Forward RNN Elapsed: %f ms\n", wl_time_forward / n_iter);
+        printf("Forward RNN time results:\n");
+        t.Print();
     }
 
     if(io_layout != miopenRNNDataSeqMajorNotPadded)
@@ -1186,9 +1166,7 @@ int RNNSeqDriver<Tgpu, Tref>::RunBackwardGPU()
 
     if((inflags.GetValueInt("forw") & 2) || (inflags.GetValueInt("forw") == 0))
     {
-        Timer t;
-        float wl_time_backward_data = 0.0;
-        float kl_time_backward_data = 0.0;
+        RNNCombTimeLoger t(GetStream(), inflags.GetValueInt("iter"), inflags.GetValueInt("wall"));
 
         workspace_dev->ToGPU(q, workspace.data());
         if(inflags.GetValueInt("inputmode") == 1)
@@ -1201,7 +1179,7 @@ int RNNSeqDriver<Tgpu, Tref>::RunBackwardGPU()
 
         for(int i = 0; i < inflags.GetValueInt("iter"); i++)
         {
-            START_TIME
+            t.Start();
             ret = miopenRNNBackwardSeqData(GetHandle(),
                                            rnnDesc,
                                            outputSeqTensor,
@@ -1223,33 +1201,16 @@ int RNNSeqDriver<Tgpu, Tref>::RunBackwardGPU()
                                            workspace_dev->GetSize(),
                                            reservespace_dev->GetMem(),
                                            reservespace_dev->GetSize());
-
-            miopen::deref(GetHandle()).Finish();
-            STOP_TIME
-            if(i > 0 || inflags.GetValueInt("iter") == 1)
-            {
-                float time = 0.0;
-                miopenGetKernelTime(GetHandle(), &time);
-                wl_time_backward_data += t.gettime_ms();
-                kl_time_backward_data += time;
-            }
-        }
-
-        if(inflags.GetValueInt("time") == 1)
-        {
-            int n_iter = inflags.GetValueInt("iter") > 1 ? inflags.GetValueInt("iter") - 1
-                                                         : inflags.GetValueInt("iter");
-            printf("GPU Kernel Time Backward Data RNN Elapsed: %f ms\n",
-                   kl_time_backward_data / n_iter);
+            t.StopAndPush();
         }
 
+        miopen::deref(GetHandle()).Finish();
         if(WALL_CLOCK)
         {
-            int n_iter = inflags.GetValueInt("iter") > 1 ? inflags.GetValueInt("iter") - 1
-                                                         : inflags.GetValueInt("iter");
-            printf("Wall-clock Time Backward Data RNN Elapsed: %f ms\n",
-                   wl_time_backward_data / n_iter);
+            printf("Backward Data RNN time results:\n");
+            t.Print();
         }
+
         if(io_layout != miopenRNNDataSeqMajorNotPadded)
         {
             auto from_gpu_din = std::vector<Tgpu>(din_dev->GetSize() / sizeof(Tgpu));
@@ -1290,13 +1251,11 @@ int RNNSeqDriver<Tgpu, Tref>::RunBackwardGPU()
 
     if((inflags.GetValueInt("forw") & 4) || (inflags.GetValueInt("forw") == 0))
     {
-        Timer t;
-        float wl_time_backward_weight = 0.0;
-        float kl_time_backward_weight = 0.0;
+        RNNCombTimeLoger t(GetStream(), inflags.GetValueInt("iter"), inflags.GetValueInt("wall"));
 
         for(int i = 0; i < inflags.GetValueInt("iter"); i++)
         {
-            START_TIME
+            t.Start();
             ret = miopenRNNBackwardWeightsSeqTensor(GetHandle(),
                                                     rnnDesc,
                                                     inputSeqTensor,
@@ -1311,31 +1270,15 @@ int RNNSeqDriver<Tgpu, Tref>::RunBackwardGPU()
                                                     workspace_dev->GetSize(),
                                                     reservespace_dev->GetMem(),
                                                     reservespace_dev->GetSize());
-            miopen::deref(GetHandle()).Finish();
-            STOP_TIME
-            if(i > 0 || inflags.GetValueInt("iter") == 1)
-            {
-                float time = 0.0;
-                miopenGetKernelTime(GetHandle(), &time);
-                wl_time_backward_weight += t.gettime_ms();
-                kl_time_backward_weight += time;
-            }
+            t.StopAndPush();
         }
 
-        if(inflags.GetValueInt("time") == 1)
-        {
-            int n_iter = inflags.GetValueInt("iter") > 1 ? inflags.GetValueInt("iter") - 1
-                                                         : inflags.GetValueInt("iter");
-            printf("GPU Kernel Time Backward Weights RNN Elapsed: %f ms\n",
-                   kl_time_backward_weight / n_iter);
-        }
+        miopen::deref(GetHandle()).Finish();
 
         if(WALL_CLOCK)
         {
-            int n_iter = inflags.GetValueInt("iter") > 1 ? inflags.GetValueInt("iter") - 1
-                                                         : inflags.GetValueInt("iter");
-            printf("Wall-clock Time Backward Weights RNN Elapsed: %f ms\n",
-                   wl_time_backward_weight / n_iter);
+            printf("Backward Weights RNN time results:\n");
+            t.Print();
         }
         dwei_dev->FromGPU(GetStream(), dwei.data());
     }
