@@ -1,10 +1,11 @@
+#include <miopen/buffer_info.hpp>
 #include <miopen/conv/invokers/impl_gemm_dynamic.hpp>
 #include <miopen/conv/data_invoke_params.hpp>
 #include <miopen/algorithm.hpp>
 #include <miopen/handle.hpp>
 #include <miopen/tensor_ops.hpp>
 #include <miopen/solver/implicitgemm_util.hpp>
-#include <miopen/util_sol.hpp>
+#include <miopen/batched_transpose_sol.hpp>
 #include <boost/any.hpp>
 
 namespace miopen {
@@ -177,9 +178,7 @@ InvokerFactory MakeImplGemmDynamicBackwardDataInvokerFactory(const ProblemDescri
         const int gemm_k_gid = k * y_dot_slice_gid[gemm_id] * x_dot_slice_gid[gemm_id];
         is_gemm_not_empty.emplace_back(gemm_k_gid > 0);
     }
-    bool need_set_zero = false;
-    if(y < stride_h || x < stride_w || dilation_h != 1 || dilation_w != 1)
-        need_set_zero = true;
+    bool need_set_zero = true;
 
     return [=](const std::vector<Kernel>& kernels) {
         const auto kernel = kernels[0];
@@ -314,13 +313,10 @@ MakeImplGemmDynamicBackwardDataInvokerFactory(const ProblemDescription& problem,
         const int gemm_k_gid = k * y_dot_slice_gid[gemm_id] * x_dot_slice_gid[gemm_id];
         is_gemm_not_empty.emplace_back(gemm_k_gid > 0);
     }
-    bool need_set_zero = false;
-    if(y < stride_h || x < stride_w || dilation_h != 1 || dilation_w != 1)
-        need_set_zero = true;
-
-    int nxb = cfg.nxb;
-    int b   = h_tilda_slice * w_tilda_slice;
-    b       = (cfg.nxe == 0) ? (b) : ((b + nxb - 1) / nxb) * nxb; // pad to nxb modulo when nxe != 0
+    bool need_set_zero = true;
+    int nxb            = cfg.nxb;
+    int b              = h_tilda_slice * w_tilda_slice;
+    b = (cfg.nxe == 0) ? (b) : ((b + nxb - 1) / nxb) * nxb; // pad to nxb modulo when nxe != 0
 
     uint32_t nb_n0          = cfg.tensor_b_cluster_lengths[2] * cfg.tensor_b_thread_lengths[2];
     uint32_t nb_n1b         = cfg.tensor_b_cluster_lengths[3] * cfg.tensor_b_thread_lengths[3];
@@ -495,7 +491,8 @@ InvokerFactory MakeImplGemmDynamicForwardXdlopsNHWCInvokerFactory(
         shift_pack_1 = 0;
     }
 
-    bool need_set_zero                 = config.gemm_k_global_split > 0;
+    // Clear buffer for all condition to resolve the NaN issue.
+    bool need_set_zero                 = true;
     bool use_fp32_global_split_on_fp16 = config.vector_store == 1 && config.gemm_k_global_split > 0;
 
     std::vector<OpKernelArg> opArgs;
@@ -539,7 +536,7 @@ InvokerFactory MakeImplGemmDynamicForwardXdlopsNHWCInvokerFactory(
         if(problem.GetOut().GetType() == miopenHalf)
             return use_fp32_global_split_on_fp16;
         if(problem.GetOut().GetType() == miopenBFloat16)
-            return need_set_zero;
+            return config.gemm_k_global_split > 0;
         return false;
     }();
     const auto is_nchw = problem.IsLayoutDefault();
@@ -561,8 +558,6 @@ InvokerFactory MakeImplGemmDynamicForwardXdlopsNHWCInvokerFactory(
     int trans_weight_idx = -1;
     int trans_output_idx = -1;
 
-    constexpr size_t buf_alignment = 256;
-
     if(is_nchw)
     {
         TransposeSolutionDefault2Nhwc trans_input(ctx, problem.GetInDataType(), n, c, hi, wi);
@@ -601,7 +596,7 @@ InvokerFactory MakeImplGemmDynamicForwardXdlopsNHWCInvokerFactory(
     const size_t cast_size = need_cast ? miopen::GetTypeSize(miopenFloat) * n * k * ho * wo : 0;
 
     MultiBufferWorkspaceTraits wt(
-        {trans_input_size, trans_weight_size, trans_output_size, cast_size}, buf_alignment);
+        {trans_input_size, trans_weight_size, trans_output_size, cast_size});
 
     trans_input_offset  = wt.GetOffset(0);
     trans_weight_offset = wt.GetOffset(1);
@@ -798,11 +793,8 @@ InvokerFactory MakeImplGemmDynamicBackwardDataXdlopsNHWCInvokerFactory(
     int dtile_h  = num_of_gemms > 1 ? static_cast<int>(mdiv_group_mn.magic) : h_tilda;
     int dtile_w  = num_of_gemms > 1 ? static_cast<int>(mdiv_group_mn.shift) : w_tilda;
 
-    bool need_set_zero                 = false;
+    bool need_set_zero                 = true;
     bool use_fp32_global_split_on_fp16 = config.vector_store == 1 && config.gemm_k_global_split > 0;
-    if(y < stride_h || x < stride_w || dilation_h != 1 || dilation_w != 1)
-        need_set_zero = true;
-    need_set_zero |= config.gemm_k_global_split > 0;
 
     std::vector<OpKernelArg> opArgs;
     opArgs.emplace_back(0); // placeholder
@@ -857,7 +849,9 @@ InvokerFactory MakeImplGemmDynamicBackwardDataXdlopsNHWCInvokerFactory(
         if(problem.GetOut().GetType() == miopenHalf)
             return use_fp32_global_split_on_fp16;
         if(problem.GetOut().GetType() == miopenBFloat16)
-            return need_set_zero;
+        {
+            return config.gemm_k_global_split > 0;
+        }
         return false;
     }();
     const auto is_nchw = problem.IsLayoutDefault();
@@ -879,8 +873,6 @@ InvokerFactory MakeImplGemmDynamicBackwardDataXdlopsNHWCInvokerFactory(
     int trans_weight_idx = -1;
     int trans_output_idx = -1;
 
-    constexpr size_t buf_alignment = 256;
-
     if(is_nchw)
     {
         TransposeSolutionNhwc2Default trans_input(ctx, problem.GetOutDataType(), n, c, hi, wi);
@@ -919,7 +911,7 @@ InvokerFactory MakeImplGemmDynamicBackwardDataXdlopsNHWCInvokerFactory(
     const size_t cast_size = need_cast ? miopen::GetTypeSize(miopenFloat) * n * c * hi * wi : 0;
 
     MultiBufferWorkspaceTraits wt(
-        {trans_input_size, trans_weight_size, trans_output_size, cast_size}, buf_alignment);
+        {trans_input_size, trans_weight_size, trans_output_size, cast_size});
 
     trans_input_offset  = wt.GetOffset(0);
     trans_weight_offset = wt.GetOffset(1);
