@@ -25,6 +25,7 @@
  *******************************************************************************/
 #include <miopen/convolution.hpp>
 
+#include <miopen/any_solver.hpp>
 #include <miopen/config.h>
 #include <miopen/env.hpp>
 #include <miopen/errors.hpp>
@@ -34,7 +35,7 @@
 #include <miopen/logger.hpp>
 #include <miopen/miopen.h>
 #include <miopen/mlo_internal.hpp>
-#include <miopen/solver.hpp>
+#include <miopen/conv/solvers.hpp>
 #include <miopen/tensor.hpp>
 #include <miopen/tensor_layout.hpp>
 #include <miopen/algorithm.hpp>
@@ -79,7 +80,7 @@ std::size_t GetWorkSpaceSizeGEMM(const miopen::ExecutionContext& ctx,
                                  const conv::ProblemDescription& problem)
 {
 #if MIOPEN_USE_GEMM
-    if(miopen::IsDisabled(ENV(MIOPEN_DEBUG_CONV_GEMM)) ||
+    if(env::disabled(MIOPEN_DEBUG_CONV_GEMM) ||
        miopen::any_of(problem.GetConv().GetConvDilations(), [](auto v) { return v > 1; }))
         return 0;
 
@@ -94,7 +95,7 @@ std::size_t GetWorkSpaceSizeGEMM(const miopen::ExecutionContext& ctx,
 std::size_t GetWorkSpaceSizeImplicitGemm(const miopen::ExecutionContext& ctx,
                                          const conv::ProblemDescription& problem)
 {
-    if(miopen::IsDisabled(ENV(MIOPEN_DEBUG_CONV_IMPLICIT_GEMM)))
+    if(env::disabled(MIOPEN_DEBUG_CONV_IMPLICIT_GEMM))
         return 0;
     return GetMaxWorkSpaceSize(FindAllImplicitGemmWorkspaceSizes(ctx, problem));
 }
@@ -102,7 +103,7 @@ std::size_t GetWorkSpaceSizeImplicitGemm(const miopen::ExecutionContext& ctx,
 std::size_t GetWorkSpaceSizeDirect(const miopen::ExecutionContext& ctx,
                                    const conv::ProblemDescription& problem)
 {
-    if(miopen::IsDisabled(ENV(MIOPEN_DEBUG_CONV_DIRECT)))
+    if(env::disabled(MIOPEN_DEBUG_CONV_DIRECT))
         return 0;
     return GetMaxWorkSpaceSize(AllDirectForwardBackwardDataWorkspaceSize(ctx, problem));
 }
@@ -110,7 +111,7 @@ std::size_t GetWorkSpaceSizeDirect(const miopen::ExecutionContext& ctx,
 std::size_t GetWorkSpaceSizeFFT(const miopen::ExecutionContext& ctx,
                                 const conv::ProblemDescription& problem)
 {
-    if(miopen::IsDisabled(ENV(MIOPEN_DEBUG_CONV_FFT)))
+    if(env::disabled(MIOPEN_DEBUG_CONV_FFT))
         return 0;
     return GetMaxWorkSpaceSize(AllFFTForwardBackwardDataWorkspaceSize(ctx, problem));
 }
@@ -118,7 +119,7 @@ std::size_t GetWorkSpaceSizeFFT(const miopen::ExecutionContext& ctx,
 std::size_t GetWorkSpaceSizeWinograd(const miopen::ExecutionContext& ctx,
                                      const conv::ProblemDescription& problem)
 {
-    if(miopen::IsDisabled(ENV(MIOPEN_DEBUG_CONV_WINOGRAD)))
+    if(env::disabled(MIOPEN_DEBUG_CONV_WINOGRAD))
         return 0;
     return GetMaxWorkSpaceSize(FindAllWinogradWorkspaceSizes(ctx, problem));
 }
@@ -126,7 +127,7 @@ std::size_t GetWorkSpaceSizeWinograd(const miopen::ExecutionContext& ctx,
 std::size_t GetWorkSpaceSizeDirectWrW(const miopen::ExecutionContext& ctx,
                                       const conv::ProblemDescription& problem)
 {
-    if(miopen::IsDisabled(ENV(MIOPEN_DEBUG_CONV_DIRECT)))
+    if(env::disabled(MIOPEN_DEBUG_CONV_DIRECT))
         return 0;
     return GetMaxWorkSpaceSize(AllDirectBwdWrW2DWorkspaceSize(ctx, problem));
 }
@@ -134,7 +135,7 @@ std::size_t GetWorkSpaceSizeDirectWrW(const miopen::ExecutionContext& ctx,
 std::size_t GetWorkSpaceSizeWinogradWrW(const miopen::ExecutionContext& ctx,
                                         const conv::ProblemDescription& problem)
 {
-    if(miopen::IsDisabled(ENV(MIOPEN_DEBUG_CONV_WINOGRAD)))
+    if(env::disabled(MIOPEN_DEBUG_CONV_WINOGRAD))
         return 0;
     return GetMaxWorkSpaceSize(FindWinogradWrWWorkspaceSizes(ctx, problem));
 }
@@ -142,7 +143,7 @@ std::size_t GetWorkSpaceSizeWinogradWrW(const miopen::ExecutionContext& ctx,
 std::size_t GetWorkSpaceSizeImplicitGemmWrW(const miopen::ExecutionContext& ctx,
                                             const conv::ProblemDescription& problem)
 {
-    if(miopen::IsDisabled(ENV(MIOPEN_DEBUG_CONV_IMPLICIT_GEMM)))
+    if(env::disabled(MIOPEN_DEBUG_CONV_IMPLICIT_GEMM))
         return 0;
     return GetMaxWorkSpaceSize(FindImplicitGemmWrWWorkspaceSizes(ctx, problem));
 }
@@ -354,7 +355,7 @@ ConvolutionDescriptor::GetForwardOutputTensorWithLayout(const TensorDescriptor&
     out_lens[0] = in_n;
     out_lens[1] = out_c;
 
-    const std::string default_layout = tensor_layout_get_default(xDesc.GetSize());
+    const std::string default_layout = tensor_layout_get_default(xDesc.GetNumDims());
     std::vector<std::size_t> out_strides;
     tensor_layout_to_strides(
         out_lens, default_layout, yLayout, xDesc.GetVectorLength(), out_strides);
@@ -372,8 +373,7 @@ TensorDescriptor ConvolutionDescriptor::GetForwardOutputTensor(const TensorDescr
                                                                miopenDataType_t yType) const
 {
     // output layout same as input
-    const std::string default_layout = tensor_layout_get_default(xDesc.GetSize());
-    const std::string in_layout      = xDesc.GetLayout(default_layout);
+    const std::string in_layout = xDesc.GetLayout_str();
     return GetForwardOutputTensorWithLayout(xDesc, wDesc, in_layout, yType);
 }
 
@@ -385,7 +385,7 @@ TensorDescriptor ConvolutionDescriptor::GetForwardOutputTensor(const TensorDescr
 bool ConvolutionDescriptor::IsWinograd3x3SupportedAndFast(
     const miopen::ExecutionContext& ctx, const conv::ProblemDescription& problem) const
 {
-    if(miopen::IsDisabled(ENV(MIOPEN_DEBUG_CONV_WINOGRAD)))
+    if(env::disabled(MIOPEN_DEBUG_CONV_WINOGRAD))
         return false;
 
     // Disable this performance optimization when we want to run some specific Solver.
@@ -424,13 +424,17 @@ std::size_t ConvolutionDescriptor::GetWorkSpaceSize(ExecutionContext ctx,
         auto fallback        = bool{};
         const auto solutions = GetSolutions(ctx, problem, 1, &fallback);
         if(solutions.empty() || ((findMode.IsHybrid(ctx) && fallback) &&
-                                 !miopen::IsEnabled(ENV(MIOPEN_DEBUG_FORCE_IMMED_MODE_FALLBACK))))
+                                 !env::enabled(MIOPEN_DEBUG_FORCE_IMMED_MODE_FALLBACK)))
         {
             ctx.use_dynamic_solutions_only = findMode.IsDynamicHybrid(ctx);
             break; // Fall down to Normal Find.
         }
-        MIOPEN_LOG_I(solutions.front().workspace_size);
-        return solutions.front().workspace_size;
+        const auto id             = solver::Id{solutions.front().solution_id};
+        const auto& s             = id.GetSolver();
+        const auto workspace_size = s.GetWorkspaceSize(ctx, problem);
+
+        MIOPEN_LOG_I(workspace_size);
+        return workspace_size;
     }
 
     size_t workspace_size;
