@@ -36,8 +36,7 @@
 
 #include <miopen/each_args.hpp>
 #include <miopen/object.hpp>
-#include <miopen/config.h>
-#include <miopen/export.h>
+#include <miopen/config.hpp>
 
 #if MIOPEN_USE_ROCTRACER
 #include <roctracer/roctx.h>
@@ -218,12 +217,14 @@ MIOPEN_EXPORT extern bool
 
 } // namespace debug
 
-MIOPEN_EXPORT const char* LoggingLevelToCString(LoggingLevel level);
-MIOPEN_EXPORT std::string LoggingPrefix();
+MIOPEN_INTERNALS_EXPORT std::string LoggingLevelToCustomString(LoggingLevel level,
+                                                               const char* custom);
+MIOPEN_INTERNALS_EXPORT const char* LoggingLevelToCString(LoggingLevel level);
+MIOPEN_INTERNALS_EXPORT std::string LoggingPrefix();
 
 /// \return true if level is enabled.
 /// \param level - one of the values defined in LoggingLevel.
-MIOPEN_EXPORT bool IsLogging(LoggingLevel level, bool disableQuieting = false);
+MIOPEN_INTERNALS_EXPORT bool IsLogging(LoggingLevel level, bool disableQuieting = false);
 bool IsLoggingCmd();
 bool IsLoggingFunctionCalls();
 #if MIOPEN_USE_ROCTRACER
@@ -347,11 +348,21 @@ LogParam(std::ostream& os, std::string name, const std::vector<T>& vec, bool ind
 #define MIOPEN_LOG_FUNCTION(...)
 #endif
 
-MIOPEN_EXPORT
-std::string LoggingParseFunction(const char* func, const char* pretty_func);
+constexpr std::string_view LoggingParseFunction(const std::string_view func,
+                                                const std::string_view pretty_func)
+{
+    if(func != "operator()")
+        return func;
+    // lambda
+    const auto pf_tail = pretty_func.substr(0, pretty_func.find_first_of('('));
+    return pf_tail.substr(1 + pf_tail.find_last_of(':'));
+}
 
-#define MIOPEN_GET_FN_NAME() \
-    (miopen::LoggingParseFunction(__func__, __PRETTY_FUNCTION__)) /* NOLINT */
+#ifdef _MSC_VER
+#define MIOPEN_GET_FN_NAME miopen::LoggingParseFunction(__func__, __FUNCSIG__)
+#else
+#define MIOPEN_GET_FN_NAME miopen::LoggingParseFunction(__func__, __PRETTY_FUNCTION__)
+#endif
 
 #define MIOPEN_LOG_XQ_CUSTOM(level, disableQuieting, category, fn_name, ...)                \
     do                                                                                      \
@@ -368,10 +379,14 @@ std::string LoggingParseFunction(const char* func, const char* pretty_func);
 #define MIOPEN_LOG_XQ_(level, disableQuieting, fn_name, ...) \
     MIOPEN_LOG_XQ_CUSTOM(level, disableQuieting, LoggingLevelToCString(level), fn_name, __VA_ARGS__)
 
-#define MIOPEN_LOG_CUSTOM(level, category, ...) \
-    MIOPEN_LOG_XQ_CUSTOM(level, false, category, MIOPEN_GET_FN_NAME(), __VA_ARGS__)
-#define MIOPEN_LOG(level, ...) MIOPEN_LOG_XQ_(level, false, MIOPEN_GET_FN_NAME(), __VA_ARGS__)
-#define MIOPEN_LOG_NQ_(level, ...) MIOPEN_LOG_XQ_(level, true, MIOPEN_GET_FN_NAME(), __VA_ARGS__)
+#define MIOPEN_LOG_CUSTOM(level, category, ...)                       \
+    MIOPEN_LOG_XQ_CUSTOM(level,                                       \
+                         false,                                       \
+                         LoggingLevelToCustomString(level, category), \
+                         MIOPEN_GET_FN_NAME,                          \
+                         __VA_ARGS__)
+#define MIOPEN_LOG(level, ...) MIOPEN_LOG_XQ_(level, false, MIOPEN_GET_FN_NAME, __VA_ARGS__)
+#define MIOPEN_LOG_NQ_(level, ...) MIOPEN_LOG_XQ_(level, true, MIOPEN_GET_FN_NAME, __VA_ARGS__)
 
 #define MIOPEN_LOG_E(...) MIOPEN_LOG(miopen::LoggingLevel::Error, __VA_ARGS__)
 #define MIOPEN_LOG_E_FROM(from, ...) \
@@ -389,18 +404,22 @@ std::string LoggingParseFunction(const char* func, const char* pretty_func);
 // Warnings in installable builds, errors otherwise.
 #define MIOPEN_LOG_WE(...) MIOPEN_LOG(LogWELevel, __VA_ARGS__)
 
-#define MIOPEN_LOG_DRIVER_CMD(...)                                                             \
-    do                                                                                         \
-    {                                                                                          \
-        std::ostringstream miopen_driver_cmd_ss;                                               \
-        miopen_driver_cmd_ss << miopen::LoggingPrefix() << "Command"                           \
-                             << " ["                                                           \
-                             << miopen::LoggingParseFunction(__func__,                         \
-                                                             __PRETTY_FUNCTION__) /* NOLINT */ \
-                             << "] ./bin/MIOpenDriver " << __VA_ARGS__ << std::endl;           \
-        std::cerr << miopen_driver_cmd_ss.str();                                               \
+#define MIOPEN_LOG_DRIVER_COMMAND(driver, ...)                                               \
+    do                                                                                       \
+    {                                                                                        \
+        std::ostringstream miopen_driver_cmd_ss;                                             \
+        miopen_driver_cmd_ss << miopen::LoggingPrefix() << "Command"                         \
+                             << " [" << MIOPEN_GET_FN_NAME << "] " driver " " << __VA_ARGS__ \
+                             << std::endl;                                                   \
+        std::cerr << miopen_driver_cmd_ss.str();                                             \
     } while(false)
 
+#ifdef _WIN32
+#define MIOPEN_LOG_DRIVER_CMD(...) MIOPEN_LOG_DRIVER_COMMAND("MIOpenDriver.exe", __VA_ARGS__)
+#else
+#define MIOPEN_LOG_DRIVER_CMD(...) MIOPEN_LOG_DRIVER_COMMAND("./bin/MIOpenDriver", __VA_ARGS__)
+#endif
+
 #if MIOPEN_LOG_FUNC_TIME_ENABLE
 class LogScopeTime
 {
@@ -421,7 +440,7 @@ private:
     std::chrono::time_point<std::chrono::high_resolution_clock> m_beg;
 };
 
-#define MIOPEN_LOG_SCOPE_TIME const miopen::LogScopeTime miopen_timer(MIOPEN_GET_FN_NAME())
+#define MIOPEN_LOG_SCOPE_TIME const miopen::LogScopeTime miopen_timer(MIOPEN_GET_FN_NAME)
 #else
 #define MIOPEN_LOG_SCOPE_TIME
 #endif
