@@ -55,7 +55,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #define NOMINMAX // stupid windows.h confused with min() macros in std namespace
 #endif
 
-#include <miopen/config.h>
+#include <miopen/config.hpp>
 
 #if MIOPEN_BACKEND_OPENCL
 #include <miopen/oclkernel.hpp>
@@ -63,11 +63,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <miopen/ocldeviceinfo.hpp>
 #endif
 #include <miopen/db_path.hpp>
-#if MIOPEN_ENABLE_SQLITE
-#include <miopen/sqlite_db.hpp>
-#else
-#include <miopen/readonlyramdb.hpp>
-#endif
+#include <miopen/conv/db_getter.hpp>
 #include <miopen/execution_context.hpp>
 #include <miopen/handle.hpp>
 #include <miopen/problem_description.hpp>
@@ -119,6 +115,8 @@ inline int AlignUp(int val, unsigned step)
     return static_cast<int>(((static_cast<unsigned>(val) + step - 1) / step) * step);
 }
 
+inline size_t AlignUp(size_t val, size_t step) { return (val + step - 1) / step * step; }
+
 namespace miopen {
 
 struct TensorDescriptor;
@@ -134,12 +132,7 @@ class DbTimer;
 
 struct AnyInvokeParams;
 
-#if MIOPEN_ENABLE_SQLITE
-using PerformanceDb = DbTimer<MultiFileDb<SQLitePerfDb, SQLitePerfDb, true>>;
-#else
-using PerformanceDb = DbTimer<MultiFileDb<ReadonlyRamDb, RamDb, true>>;
-#endif
-miopen::PerformanceDb GetDb(const miopen::ExecutionContext& ctx);
+MIOPEN_INTERNALS_EXPORT miopen::PerformanceDb GetDb(const miopen::ExecutionContext& ctx);
 
 template <class TTo>
 size_t setTopDescFromMLDesc(int spatial_dims, TTo& to, const TensorDescriptor& tensor)
@@ -179,74 +172,74 @@ auto mloConstruct(T& x) -> decltype(x.mloConstruct(), void())
     x.mloConstruct();
 }
 
-std::vector<miopen::solver::ConvSolution>
+MIOPEN_INTERNALS_EXPORT std::vector<miopen::solver::ConvSolution>
 FindAllGemmSolutions(const miopen::ExecutionContext& ctx,
                      const miopen::conv::ProblemDescription& problem,
                      const miopen::AnyInvokeParams& invoke_ctx);
 
-std::vector<std::pair<std::string, size_t>>
+MIOPEN_INTERNALS_EXPORT std::vector<std::pair<std::string, size_t>>
 AllGemmWorkspaceSize(const miopen::ExecutionContext& ctx,
                      const miopen::conv::ProblemDescription& problem);
 
-std::vector<std::pair<std::string, size_t>>
+MIOPEN_INTERNALS_EXPORT std::vector<std::pair<std::string, size_t>>
 AllDirectForwardBackwardDataWorkspaceSize(const miopen::ExecutionContext& ctx,
                                           const miopen::conv::ProblemDescription& problem);
 
-std::vector<std::pair<std::string, size_t>>
+MIOPEN_INTERNALS_EXPORT std::vector<std::pair<std::string, size_t>>
 FindAllImplicitGemmWorkspaceSizes(const miopen::ExecutionContext& ctx,
                                   const miopen::conv::ProblemDescription& problem);
 
-std::vector<std::pair<std::string, size_t>>
+MIOPEN_INTERNALS_EXPORT std::vector<std::pair<std::string, size_t>>
 FindAllWinogradWorkspaceSizes(const miopen::ExecutionContext& ctx,
                               const miopen::conv::ProblemDescription& problem);
 
-std::vector<std::pair<std::string, size_t>>
+MIOPEN_INTERNALS_EXPORT std::vector<std::pair<std::string, size_t>>
 FindWinogradWrWWorkspaceSizes(const miopen::ExecutionContext& ctx,
                               const miopen::conv::ProblemDescription& problem);
 
-std::vector<std::pair<std::string, size_t>>
+MIOPEN_INTERNALS_EXPORT std::vector<std::pair<std::string, size_t>>
 FindImplicitGemmWrWWorkspaceSizes(const miopen::ExecutionContext& ctx,
                                   const miopen::conv::ProblemDescription& problem);
 
-std::vector<std::pair<std::string, size_t>>
+MIOPEN_INTERNALS_EXPORT std::vector<std::pair<std::string, size_t>>
 AllDirectBwdWrW2DWorkspaceSize(const miopen::ExecutionContext& ctx,
                                const miopen::conv::ProblemDescription& problem);
 
-std::vector<std::pair<std::string, size_t>>
+MIOPEN_INTERNALS_EXPORT std::vector<std::pair<std::string, size_t>>
 AllFFTForwardBackwardDataWorkspaceSize(const miopen::ExecutionContext& ctx,
                                        const miopen::conv::ProblemDescription& problem);
 
-std::vector<miopen::solver::ConvSolution>
+MIOPEN_INTERNALS_EXPORT std::vector<miopen::solver::ConvSolution>
 FindAllDirectSolutions(const miopen::ExecutionContext& ctx,
                        const miopen::conv::ProblemDescription& problem,
                        const miopen::AnyInvokeParams& invoke_ctx);
 
-std::vector<miopen::solver::ConvSolution>
+MIOPEN_INTERNALS_EXPORT std::vector<miopen::solver::ConvSolution>
 FindAllImplicitGemmSolutions(const miopen::ExecutionContext& ctx,
                              const miopen::conv::ProblemDescription& problem,
                              const miopen::AnyInvokeParams& invoke_ctx);
 
-std::vector<miopen::solver::ConvSolution>
+MIOPEN_INTERNALS_EXPORT std::vector<miopen::solver::ConvSolution>
 FindAllWinogradSolutions(const miopen::ExecutionContext& ctx,
                          const miopen::conv::ProblemDescription& problem,
                          const miopen::AnyInvokeParams& invoke_ctx);
 
-std::vector<miopen::solver::ConvSolution>
+MIOPEN_INTERNALS_EXPORT std::vector<miopen::solver::ConvSolution>
 FindWinogradWrWAllSolutions(const miopen::ExecutionContext& ctx,
                             const miopen::conv::ProblemDescription& problem,
                             const miopen::AnyInvokeParams& invoke_ctx);
 
-std::vector<miopen::solver::ConvSolution>
+MIOPEN_INTERNALS_EXPORT std::vector<miopen::solver::ConvSolution>
 FindImplicitGemmWrWAllSolutions(const miopen::ExecutionContext& ctx,
                                 const miopen::conv::ProblemDescription& problem,
                                 const miopen::AnyInvokeParams& invoke_ctx);
 
-std::vector<miopen::solver::ConvSolution>
+MIOPEN_INTERNALS_EXPORT std::vector<miopen::solver::ConvSolution>
 FindAllBwdWrW2DSolutions(const miopen::ExecutionContext& ctx,
                          const miopen::conv::ProblemDescription& problem,
                          const miopen::AnyInvokeParams& invoke_ctx);
 
-std::vector<miopen::solver::ConvSolution>
+MIOPEN_INTERNALS_EXPORT std::vector<miopen::solver::ConvSolution>
 FindAllFFTSolutions(const miopen::ExecutionContext& ctx,
                     const miopen::conv::ProblemDescription& problem,
                     const miopen::AnyInvokeParams& invoke_ctx);
@@ -272,7 +265,7 @@ struct mlo_construct_base
     /*
      * set library stream
      */
-    inline void setStream(miopen::Handle* stream) { _ctx.SetStream(stream); }
+    inline void setStream(const miopen::Handle* stream) { _ctx.SetStream(stream); }
 
 protected:
     miopen::ProblemDescriptionCompatTemporary _problem;
