@@ -65,7 +65,7 @@ struct c_array_view
 
 void profileRNNkernels(const Handle& handle, unsigned char select, float& ctime);
 
-struct RNNDescriptor : miopenRNNDescriptor
+struct MIOPEN_INTERNALS_EXPORT RNNDescriptor : miopenRNNDescriptor
 {
 
     RNNDescriptor();
@@ -121,6 +121,11 @@ struct RNNDescriptor : miopenRNNDescriptor
                                                        const int* lensPerSeq,
                                                        const void* padding_marker_ptr);
 
+    static SeqTensorDescriptor makeSeqTensorDescriptor(
+        c_array_view<const miopenTensorDescriptor_t> descs,
+        size_t seq_len,
+        miopenRNNBaseLayout_t layout = miopenRNNBaseLayout_t::miopenRNNDataSeqMajorNotPadded);
+
     static void SeqTensorToTensorDescArray(const SeqTensorDescriptor& desc,
                                            std::vector<miopen::TensorDescriptor>& td,
                                            std::vector<miopenTensorDescriptor_t>& ptd);
@@ -133,36 +138,46 @@ struct RNNDescriptor : miopenRNNDescriptor
                                    miopenRNNFWDMode_t fwdMode,
                                    miopenRNNBaseLayout_t ioLayout) const;
 
-    size_t GetWorkspaceSize(Handle& handle,
+    size_t GetWorkspaceSize(const Handle& handle,
                             int seqLength,
                             c_array_view<const miopenTensorDescriptor_t> xDesc) const;
-    size_t GetWorkspaceSize(Handle& handle,
+    size_t GetWorkspaceSize(const Handle& handle,
                             const SeqTensorDescriptor& xDesc,
                             miopenRNNFWDMode_t fwdMode) const;
 
     size_t GetReserveSize(size_t batchLenSum) const;
-    size_t GetReserveSize(Handle& handle,
+    size_t GetReserveSize(const Handle& handle,
                           int seqLength,
                           c_array_view<const miopenTensorDescriptor_t> xDesc) const;
 
-    size_t GetMaxWorkspaceSize(Handle& handle,
+    size_t GetMaxWorkspaceSize(const Handle& handle,
                                const SeqTensorDescriptor& xDesc,
                                miopenRNNFWDMode_t fwdMode) const;
-    size_t GetMaxReserveSize(Handle& handle, const SeqTensorDescriptor& xDesc) const;
+    size_t GetMaxReserveSize(const Handle& handle, const SeqTensorDescriptor& xDesc) const;
 
-    size_t
-    GetParamsSize(Handle& handle, const TensorDescriptor& xDesc, miopenDataType_t dtype) const;
+    std::tuple<size_t, size_t> GetTmpSpaceSizeDynamicAlgo(const Handle& handle,
+                                                          const SeqTensorDescriptor& xDesc,
+                                                          miopenRNNFWDMode_t fwdMode) const;
+    bool CheckDynamicAlgoSelection(const Handle& handle,
+                                   const SeqTensorDescriptor& xDesc,
+                                   miopenRNNFWDMode_t fwdMode) const;
+
+    size_t GetParamsSize(const Handle& handle,
+                         const TensorDescriptor& xDesc,
+                         miopenDataType_t dtype) const;
     size_t GetParamsSize(size_t inputVector) const;
 
-    void GetParamsDescriptor(Handle& handle,
+    void GetParamsDescriptor(const Handle& handle,
                              const TensorDescriptor& xDesc,
                              TensorDescriptor& wDesc,
                              miopenDataType_t dtype) const;
 
-    std::size_t
-    GetLayerParamSize(Handle& handle, int layer, const TensorDescriptor& xDesc, int paramID) const;
+    std::size_t GetLayerParamSize(const Handle& handle,
+                                  int layer,
+                                  const TensorDescriptor& xDesc,
+                                  int paramID) const;
 
-    std::size_t GetLayerBiasSize(Handle& handle, int layer, int biasID) const;
+    std::size_t GetLayerBiasSize(const Handle& handle, int layer, int biasID) const;
 
     void GetLayerParam(const Handle& handle,
                        int layer,
@@ -214,14 +229,14 @@ struct RNNDescriptor : miopenRNNDescriptor
                             TensorDescriptor& biasDesc,
                             size_t* biasOffset) const;
 
-    size_t GetRNNInputSuperTensorSize(Handle& handle,
+    size_t GetRNNInputSuperTensorSize(const Handle& handle,
                                       int seqLength,
                                       c_array_view<miopenTensorDescriptor_t> xDesc) const;
 
-    size_t GetRNNHiddenSuperTensorSize(Handle& handle,
+    size_t GetRNNHiddenSuperTensorSize(const Handle& handle,
                                        c_array_view<miopenTensorDescriptor_t> xDesc) const;
 
-    void RNNForward(Handle& handle,
+    void RNNForward(const Handle& handle,
                     miopenRNNFWDMode_t fwdMode,
                     const SeqTensorDescriptor& xDesc,
                     ConstData_t x,
@@ -240,7 +255,7 @@ struct RNNDescriptor : miopenRNNDescriptor
                     Data_t reserveSpace,
                     size_t reserveSpaceSize) const;
 
-    void RNNForwardTraining(Handle& handle,
+    void RNNForwardTraining(const Handle& handle,
                             int seqLen,
                             c_array_view<const miopenTensorDescriptor_t> xDesc,
                             ConstData_t x,
@@ -261,7 +276,7 @@ struct RNNDescriptor : miopenRNNDescriptor
                             Data_t reserveSpace,
                             size_t reserveSpaceSize) const;
 
-    void RNNForwardInference(Handle& handle,
+    void RNNForwardInference(const Handle& handle,
                              int seqLen,
                              c_array_view<const miopenTensorDescriptor_t> xDesc,
                              ConstData_t x,
@@ -280,7 +295,7 @@ struct RNNDescriptor : miopenRNNDescriptor
                              Data_t workSpace,
                              size_t workSpaceSize) const;
 
-    void RNNBackwardData(Handle& handle,
+    void RNNBackwardData(const Handle& handle,
                          const SeqTensorDescriptor& yDesc,
                          ConstData_t y,
                          ConstData_t dy,
@@ -301,7 +316,7 @@ struct RNNDescriptor : miopenRNNDescriptor
                          Data_t reserveSpace,
                          size_t reserveSpaceSize) const;
 
-    void RNNBackwardData(Handle& handle,
+    void RNNBackwardData(const Handle& handle,
                          int seqLen,
                          c_array_view<const miopenTensorDescriptor_t> yDesc,
                          ConstData_t y,
@@ -328,7 +343,7 @@ struct RNNDescriptor : miopenRNNDescriptor
                          Data_t reserveSpace,
                          size_t reserveSpaceSize) const;
 
-    void RNNBackwardWeights(Handle& handle,
+    void RNNBackwardWeights(const Handle& handle,
                             const SeqTensorDescriptor& xDesc,
                             ConstData_t x,
                             const TensorDescriptor& hDesc,
@@ -342,7 +357,7 @@ struct RNNDescriptor : miopenRNNDescriptor
                             ConstData_t reserveSpace,
                             size_t reserveSpaceSize) const;
 
-    void RNNBackwardWeights(Handle& handle,
+    void RNNBackwardWeights(const Handle& handle,
                             int seqLen,
                             c_array_view<const miopenTensorDescriptor_t> xDesc,
                             ConstData_t x,
@@ -364,7 +379,58 @@ private:
     size_t RNNTransformerWorkspaceSize(const SeqTensorDescriptor& xDesc,
                                        miopenRNNFWDMode_t fwdMode) const;
 
-    void RNNTransformerForward(Handle& handle,
+    // TODO rename
+
+    void ModularForward(const Handle& handle,
+                        miopenRNNFWDMode_t fwdMode,
+                        ConstData_t w,
+                        const SeqTensorDescriptor& xDesc,
+                        ConstData_t x,
+                        const TensorDescriptor& hDesc,
+                        ConstData_t hx,
+                        Data_t hy,
+                        const TensorDescriptor& cDesc,
+                        ConstData_t cx,
+                        Data_t cy,
+                        const SeqTensorDescriptor& yDesc,
+                        Data_t y,
+                        Data_t workSpace,
+                        size_t workSpaceSize,
+                        Data_t reserveSpace,
+                        size_t reserveSpaceSize) const;
+
+    void ModularBackward(const Handle& handle,
+                         const SeqTensorDescriptor& yDesc,
+                         ConstData_t dy,
+                         const TensorDescriptor& hDesc,
+                         ConstData_t hx,
+                         ConstData_t dhy,
+                         Data_t dhx,
+                         const TensorDescriptor& cDesc,
+                         ConstData_t cx,
+                         ConstData_t dcy,
+                         Data_t dcx,
+                         const SeqTensorDescriptor& xDesc,
+                         Data_t dx,
+                         ConstData_t w,
+                         Data_t workSpace,
+                         size_t workSpaceSize,
+                         Data_t reserveSpace,
+                         size_t reserveSpaceSize) const;
+
+    void ModularBackwardWeights(const Handle& handle,
+                                const SeqTensorDescriptor& xDesc,
+                                ConstData_t x,
+                                const TensorDescriptor& hDesc,
+                                ConstData_t hx,
+                                const SeqTensorDescriptor& yDesc,
+                                Data_t w,
+                                Data_t workSpace,
+                                size_t workSpaceSize,
+                                ConstData_t reserveSpace,
+                                size_t /*reserveSpaceSize*/) const;
+
+    void RNNTransformerForward(const Handle& handle,
                                miopenRNNFWDMode_t fwdMode,
                                ConstData_t w,
                                const SeqTensorDescriptor& xDesc,
@@ -382,7 +448,7 @@ private:
                                Data_t reserveSpace,
                                size_t reserveSpaceSize) const;
 
-    void RNNTransformerBackwardData(Handle& handle,
+    void RNNTransformerBackwardData(const Handle& handle,
                                     const SeqTensorDescriptor& yDesc,
                                     ConstData_t dy,
                                     const TensorDescriptor& hDesc,
@@ -401,7 +467,7 @@ private:
                                     Data_t reserveSpace,
                                     size_t reserveSpaceSize) const;
 
-    void RNNTransformerBackwardWeights(Handle& handle,
+    void RNNTransformerBackwardWeights(const Handle& handle,
                                        const SeqTensorDescriptor& xDesc,
                                        ConstData_t x,
                                        const TensorDescriptor& hDesc,
@@ -413,7 +479,7 @@ private:
                                        ConstData_t reserveSpace,
                                        size_t reserveSpaceSize) const;
 
-    void RNNVanillaForward(Handle& handle,
+    void RNNVanillaForward(const Handle& handle,
                            miopenRNNFWDMode_t fwdMode,
                            ConstData_t w,
                            const SeqTensorDescriptor& xDesc,
@@ -431,7 +497,7 @@ private:
                            Data_t reserveSpace,
                            size_t reserveSpaceSize) const;
 
-    void RNNVanillaBackwardData(Handle& handle,
+    void RNNVanillaBackwardData(const Handle& handle,
                                 const SeqTensorDescriptor& yDesc,
                                 ConstData_t dy,
                                 const TensorDescriptor& hDesc,
@@ -450,7 +516,7 @@ private:
                                 Data_t reserveSpace,
                                 size_t reserveSpaceSize) const;
 
-    void RNNVanillaBackwardWeights(Handle& handle,
+    void RNNVanillaBackwardWeights(const Handle& handle,
                                    const SeqTensorDescriptor& xDesc,
                                    ConstData_t x,
                                    const TensorDescriptor& hDesc,
@@ -462,7 +528,7 @@ private:
                                    ConstData_t reserveSpace,
                                    size_t reserveSpaceSize) const;
 
-    void RNNForwardTrainingPackedTensors(Handle& handle,
+    void RNNForwardTrainingPackedTensors(const Handle& handle,
                                          int seqLen,
                                          c_array_view<const miopenTensorDescriptor_t> xDesc,
                                          ConstData_t x,
@@ -478,26 +544,29 @@ private:
                                          Data_t hy,
                                          const TensorDescriptor& cyDesc,
                                          Data_t cy,
+                                         Data_t workSpace,
+                                         size_t workSpaceSize,
                                          Data_t reserveSpace,
                                          size_t reserveSpaceSize) const;
 
-    void RNNForwardTraining_MS(Handle& handle,
-                               std::vector<int>& seq_array,
-                               const TensorDescriptor& xDesc,
-                               ConstData_t x,
-                               const TensorDescriptor& hxDesc,
-                               ConstData_t hx,
-                               ConstData_t cx,
-                               const TensorDescriptor& wDesc,
-                               ConstData_t w,
-                               const TensorDescriptor& yDesc,
-                               Data_t y,
-                               Data_t hy,
-                               Data_t cy,
-                               Data_t reserveSpace,
-                               size_t reserveSpaceSize) const;
+    void RNNForwardMS(const Handle& handle,
+                      std::vector<int>& seq_array,
+                      const TensorDescriptor& xDesc,
+                      ConstData_t x,
+                      const TensorDescriptor& hxDesc,
+                      ConstData_t hx,
+                      ConstData_t cx,
+                      const TensorDescriptor& wDesc,
+                      ConstData_t w,
+                      const TensorDescriptor& yDesc,
+                      Data_t y,
+                      Data_t hy,
+                      Data_t cy,
+                      Data_t extra_space,
+                      size_t extra_space_size,
+                      miopenRNNFWDMode_t fwd_mode) const;
 
-    void RNNForwardInferencePacked(Handle& handle,
+    void RNNForwardInferencePacked(const Handle& handle,
                                    int seqLen,
                                    c_array_view<const miopenTensorDescriptor_t> xDesc,
                                    ConstData_t x,
@@ -516,7 +585,7 @@ private:
                                    Data_t workSpace,
                                    size_t workSpaceSize) const;
 
-    void RNNBackwardDataPackedTensors(Handle& handle,
+    void RNNBackwardDataPackedTensors(const Handle& handle,
                                       int seqLen,
                                       c_array_view<const miopenTensorDescriptor_t> dyDesc,
                                       ConstData_t dy,
@@ -536,7 +605,7 @@ private:
                                       Data_t reserveSpace,
                                       size_t reserveSpaceSize) const;
 
-    void RNNBackwardWeightsPackedTensors(Handle& handle,
+    void RNNBackwardWeightsPackedTensors(const Handle& handle,
                                          int seqLen,
                                          c_array_view<const miopenTensorDescriptor_t> xDesc,
                                          ConstData_t x,
@@ -551,7 +620,7 @@ private:
                                          size_t reserveSpaceSize) const;
 };
 
-std::ostream& operator<<(std::ostream& stream, const RNNDescriptor& r);
+MIOPEN_INTERNALS_EXPORT std::ostream& operator<<(std::ostream& stream, const RNNDescriptor& r);
 
 } // namespace miopen
 MIOPEN_DEFINE_OBJECT(miopenRNNDescriptor, miopen::RNNDescriptor);
