@@ -38,6 +38,8 @@
 #include <algorithm>
 #include <iostream>
 
+#include "../../test/dropout_util.hpp"
+
 #define DROPOUT_DEBUG 0
 
 namespace miopen {
@@ -117,7 +119,7 @@ inline void SquashPairedTensor(const std::vector<T> x_len,
     }
 }
 
-void DropoutDescriptor::InitPRNGState(Handle& handle,
+void DropoutDescriptor::InitPRNGState(const Handle& handle,
                                       Data_t prng_states,
                                       size_t prng_stateSizeInBytes,
                                       unsigned long long prng_seed) const
@@ -125,19 +127,19 @@ void DropoutDescriptor::InitPRNGState(Handle& handle,
 #if DROPOUT_DEBUG
     std::cout << "Check memory and threads info of dropout PRNG states in debug mode:" << std::endl;
 #endif
-    std::string program_name = "MIOpenDropout.cl";
-    std::string kernel_name  = "InitKernelState";
+    std::string program_name = "MIOpenDropoutHIP.cpp";
+    std::string kernel_name  = "InitKernelStateHIP";
 
     if(prng_stateSizeInBytes > handle.GetMaxMemoryAllocSize())
     {
         MIOPEN_THROW("PRNG state size should not exceed system maximum memory allocation size.");
     }
 
-    unsigned long long states_num = prng_stateSizeInBytes / sizeof(prngStates);
+    unsigned long long states_num = prng_stateSizeInBytes / sizeof(rocrand_state_xorwow);
     size_t wk_grp_num =
         std::min(static_cast<unsigned long long>(MAX_PRNG_STATE / 256), (states_num + 255) / 256);
 
-    std::string network_config = "initprngs-" + std::to_string(sizeof(prngStates)) + "x" +
+    std::string network_config = "initprngs-" + std::to_string(sizeof(rocrand_state_xorwow)) + "x" +
                                  std::to_string(rng_mode) + "x" + std::to_string(wk_grp_num);
 
     auto&& kernels = handle.GetKernels(kernel_name, network_config);
@@ -151,7 +153,7 @@ void DropoutDescriptor::InitPRNGState(Handle& handle,
         const std::vector<size_t> vgd{wk_grp_num * 256, 1, 1};
 
         std::string params;
-        params += " -DRUN_INIT_PRNG=1";
+        params += "-DRUN_FORWARD=0  -DRUN_INIT_PRNG=1";
 #if DROPOUT_DEBUG
         std::cout << "Threads allocated for PRNG states: " << vgd[0] << std::endl;
         std::cout << "Memory allocated for PRNG states: " << stateSizeInBytes << std::endl;
@@ -164,29 +166,30 @@ void DropoutDescriptor::InitPRNGState(Handle& handle,
     }
 }
 
-void DropoutDescriptor::DropoutForward(const Handle& handle,
-                                       const TensorDescriptor& noise_shape,
-                                       const TensorDescriptor& xDesc,
-                                       ConstData_t x,
-                                       const TensorDescriptor& yDesc,
-                                       Data_t y,
-                                       Data_t reserveSpace,
-                                       size_t reserveSpaceSizeInBytes,
-                                       size_t in_offset,
-                                       size_t out_offset,
-                                       size_t rsvsp_offset) const
+void DropoutDescriptor::Dropout(const Handle& handle,
+                                const TensorDescriptor& noise_shape,
+                                const TensorDescriptor& xDesc,
+                                ConstData_t x,
+                                const TensorDescriptor& yDesc,
+                                Data_t y,
+                                Data_t reserveSpace,
+                                size_t reserveSpaceSizeInBytes,
+                                size_t in_offset,
+                                size_t out_offset,
+                                size_t rsvsp_offset,
+                                bool is_backward) const
 {
     if(x == nullptr || y == nullptr)
     {
         MIOPEN_THROW(miopenStatusBadParm);
     }
 
-    if(xDesc.GetSize() != yDesc.GetSize())
+    if(xDesc.GetNumDims() != yDesc.GetNumDims())
     {
         MIOPEN_THROW("Input/Output dimension does not match");
     }
 
-    if(xDesc.GetSize() > 5)
+    if(xDesc.GetNumDims() > 5)
     {
         MIOPEN_THROW("Only support 1D to 5D tensors");
     }
@@ -197,7 +200,7 @@ void DropoutDescriptor::DropoutForward(const Handle& handle,
     }
 
     if(xDesc.GetElementSize() != noise_shape.GetElementSize() ||
-       xDesc.GetSize() != noise_shape.GetSize())
+       xDesc.GetNumDims() != noise_shape.GetNumDims())
     {
         MIOPEN_THROW("Only support dropout with regular noise shape currently");
     }
@@ -213,6 +216,7 @@ void DropoutDescriptor::DropoutForward(const Handle& handle,
     }
 
     bool use_rsvsp = !(reserveSpace == nullptr);
+    bool use_prng  = reserveSpace == nullptr;
     if(((use_rsvsp || use_mask) &&
         reserveSpaceSizeInBytes < xDesc.GetElementSize() * sizeof(bool)) ||
        (use_mask && reserveSpace == nullptr))
@@ -220,7 +224,7 @@ void DropoutDescriptor::DropoutForward(const Handle& handle,
         MIOPEN_THROW("Insufficient reservespace size");
     }
 
-    if(stateSizeInBytes + reserveSpaceSizeInBytes +
+    if((is_backward ? 0 : stateSizeInBytes) + reserveSpaceSizeInBytes +
            xDesc.GetElementSize() * GetTypeSize(xDesc.GetType()) +
            yDesc.GetElementSize() * GetTypeSize(yDesc.GetType()) >
        handle.GetGlobalMemorySize())
@@ -259,25 +263,48 @@ void DropoutDescriptor::DropoutForward(const Handle& handle,
         std::min(max_wk_grp / 256,
                  ((in_len[4] * in_len[3] * in_len[2] * in_len[1] * in_len[0] + 255) / 256));
 
-    size_t states_num = stateSizeInBytes / sizeof(prngStates);
+    size_t states_num = stateSizeInBytes / sizeof(rocrand_state_xorwow);
     if(states_num < wk_grp_num * 256 && !use_mask)
     {
         MIOPEN_THROW("Insufficient state size for parallel PRNG");
     }
 
-    std::string program_name = "MIOpenDropout.cl";
-    std::string kernel_name  = "DropoutForward";
+    std::string program_name   = "MIOpenDropoutHIP.cpp";
+    std::string kernel_name    = "DropoutKernel";
+    std::string network_config = "";
+    if(is_backward)
+    {
+        network_config =
+            "bwd-" + std::string(yDesc.GetType() == miopenHalf ? "fp16-" : "fp32-") + "-seed" +
+            std::to_string(seed) + "-rng" + std::to_string(rng_mode) + "-prng" +
+            std::to_string(static_cast<int>(use_prng)) + "-evo" +
+            std::to_string(static_cast<int>(state_evo)) + "-blk" + std::to_string(RD_BLCK) + "-wg" +
+            std::to_string(
+                wk_grp_num) /* + "-noise" + std::to_string(noise_shape.GetLengths()[0]) */;
+    }
+    else
+    {
+        network_config =
+            "fwd-" + std::string(xDesc.GetType() == miopenHalf ? "fp16-" : "fp32-") + "-seed" +
+            std::to_string(seed) + "-rng" + std::to_string(rng_mode) + "-rsvsp" +
+            std::to_string(static_cast<int>(use_rsvsp)) + "-mask" +
+            std::to_string(static_cast<int>(use_mask)) + "-evo" +
+            std::to_string(static_cast<int>(state_evo)) + "-blk" + std::to_string(RD_BLCK) + "-wg" +
+            std::to_string(
+                wk_grp_num) /* + "-noise" + std::to_string(noise_shape.GetLengths()[0])*/;
+    }
 
-    std::string network_config =
-        "fwd-" + std::string(xDesc.GetType() == miopenHalf ? "fp16-" : "fp32-") + "-seed" +
-        std::to_string(seed) + "-rng" + std::to_string(rng_mode) + "-rsvsp" +
-        std::to_string(static_cast<int>(use_rsvsp)) + "-mask" +
-        std::to_string(static_cast<int>(use_mask)) + "-evo" +
-        std::to_string(static_cast<int>(state_evo)) + "-blk" + std::to_string(RD_BLCK) + "-wg" +
-        std::to_string(wk_grp_num) /* + "-noise" + std::to_string(noise_shape.GetLengths()[0])*/;
+    if(xDesc.AllDimsFitIntoInt())
+    {
+        network_config += "-32bit";
+    }
+    else
+    {
+        network_config += "-64bit";
+    }
 
     // TODO: Add noise shape
-    // for(int i = 1; i < noise_shape.GetSize(); i++)
+    // for(int i = 1; i < noise_shape.GetNumDims(); i++)
     //    network_config += "x" + std::to_string(noise_shape.GetLengths()[i]);
 
     auto&& kernels = handle.GetKernels(kernel_name, network_config);
@@ -285,28 +312,56 @@ void DropoutDescriptor::DropoutForward(const Handle& handle,
     float amp_scale = float_equal(dropout, 1.0) ? 0 : 1 / (1 - dropout);
     if(!kernels.empty())
     {
-        kernels.front()(pstates,
-                        dropout,
-                        amp_scale,
-                        static_cast<int>(in_len[1]),
-                        static_cast<int>(in_len[2]),
-                        static_cast<int>(in_len[3]),
-                        static_cast<int>(in_len[4]),
-                        y,
-                        static_cast<int>(out_str[0]),
-                        static_cast<int>(out_str[1]),
-                        static_cast<int>(out_str[2]),
-                        static_cast<int>(out_str[3]),
-                        x,
-                        static_cast<int>(in_str[0]),
-                        static_cast<int>(in_str[1]),
-                        static_cast<int>(in_str[2]),
-                        static_cast<int>(in_str[3]),
-                        reserveSpace,
-                        static_cast<unsigned>(total_work),
-                        static_cast<unsigned>(in_offset),
-                        static_cast<unsigned>(out_offset),
-                        static_cast<unsigned>(rsvsp_offset));
+        if(xDesc.AllDimsFitIntoInt())
+        {
+            kernels.front()(pstates,
+                            dropout,
+                            amp_scale,
+                            static_cast<uint32_t>(in_len[1]),
+                            static_cast<uint32_t>(in_len[2]),
+                            static_cast<uint32_t>(in_len[3]),
+                            static_cast<uint32_t>(in_len[4]),
+                            y,
+                            static_cast<uint32_t>(out_str[0]),
+                            static_cast<uint32_t>(out_str[1]),
+                            static_cast<uint32_t>(out_str[2]),
+                            static_cast<uint32_t>(out_str[3]),
+                            x,
+                            static_cast<uint32_t>(in_str[0]),
+                            static_cast<uint32_t>(in_str[1]),
+                            static_cast<uint32_t>(in_str[2]),
+                            static_cast<uint32_t>(in_str[3]),
+                            reserveSpace,
+                            static_cast<uint32_t>(total_work),
+                            static_cast<uint64_t>(in_offset),
+                            static_cast<uint64_t>(out_offset),
+                            static_cast<uint64_t>(rsvsp_offset));
+        }
+        else
+        {
+            kernels.front()(pstates,
+                            dropout,
+                            amp_scale,
+                            static_cast<uint64_t>(in_len[1]),
+                            static_cast<uint64_t>(in_len[2]),
+                            static_cast<uint64_t>(in_len[3]),
+                            static_cast<uint64_t>(in_len[4]),
+                            y,
+                            static_cast<uint64_t>(out_str[0]),
+                            static_cast<uint64_t>(out_str[1]),
+                            static_cast<uint64_t>(out_str[2]),
+                            static_cast<uint64_t>(out_str[3]),
+                            x,
+                            static_cast<uint64_t>(in_str[0]),
+                            static_cast<uint64_t>(in_str[1]),
+                            static_cast<uint64_t>(in_str[2]),
+                            static_cast<uint64_t>(in_str[3]),
+                            reserveSpace,
+                            static_cast<uint64_t>(total_work),
+                            static_cast<uint64_t>(in_offset),
+                            static_cast<uint64_t>(out_offset),
+                            static_cast<uint64_t>(rsvsp_offset));
+        }
     }
     else
     {
@@ -322,42 +377,90 @@ void DropoutDescriptor::DropoutForward(const Handle& handle,
                               : RD_BLCK == 2 ? "ushort"
                                              : "uchar");
 
+        if(xDesc.AllDimsFitIntoInt())
+        {
+            params += " -DDIM_TYPE=uint32_t";
+        }
+        else
+        {
+            params += " -DDIM_TYPE=uint64_t";
+        }
+
         if(xDesc.GetType() == miopenHalf)
             params += " -DMIOPEN_USE_FP16=1";
         else
             params += " -DMIOPEN_USE_FP32=1";
 
-        params += " -DRUN_FORWARD=1";
+        // params += " -DRUN_FORWARD=1";
 
-        params += " -DUSE_RSVSP=" + std::to_string(static_cast<size_t>(use_rsvsp));
-        params += " -DUSE_MASK=" + std::to_string(static_cast<size_t>(use_mask));
+        if(is_backward)
+        {
+            params +=
+                " -DUSE_MASK=" + std::to_string(static_cast<size_t>(!use_prng)) + " -DUSE_RSVSP=0";
+        }
+        else
+        {
+            params += " -DUSE_RSVSP=" + std::to_string(static_cast<size_t>(use_rsvsp));
+            params += " -DUSE_MASK=" + std::to_string(static_cast<size_t>(use_mask));
+        }
 
         const std::vector<size_t> vld{256, 1, 1};
         const std::vector<size_t> vgd{wk_grp_num * 256, 1, 1};
 
-        handle.AddKernel(kernel_name, network_config, program_name, kernel_name, vld, vgd, params)(
-            pstates,
-            dropout,
-            amp_scale,
-            static_cast<int>(in_len[1]),
-            static_cast<int>(in_len[2]),
-            static_cast<int>(in_len[3]),
-            static_cast<int>(in_len[4]),
-            y,
-            static_cast<int>(out_str[0]),
-            static_cast<int>(out_str[1]),
-            static_cast<int>(out_str[2]),
-            static_cast<int>(out_str[3]),
-            x,
-            static_cast<int>(in_str[0]),
-            static_cast<int>(in_str[1]),
-            static_cast<int>(in_str[2]),
-            static_cast<int>(in_str[3]),
-            reserveSpace,
-            static_cast<unsigned>(total_work),
-            static_cast<unsigned>(in_offset),
-            static_cast<unsigned>(out_offset),
-            static_cast<unsigned>(rsvsp_offset));
+        if(xDesc.AllDimsFitIntoInt())
+        {
+            handle.AddKernel(
+                kernel_name, network_config, program_name, kernel_name, vld, vgd, params)(
+                pstates,
+                dropout,
+                amp_scale,
+                static_cast<uint32_t>(in_len[1]),
+                static_cast<uint32_t>(in_len[2]),
+                static_cast<uint32_t>(in_len[3]),
+                static_cast<uint32_t>(in_len[4]),
+                y,
+                static_cast<uint32_t>(out_str[0]),
+                static_cast<uint32_t>(out_str[1]),
+                static_cast<uint32_t>(out_str[2]),
+                static_cast<uint32_t>(out_str[3]),
+                x,
+                static_cast<uint32_t>(in_str[0]),
+                static_cast<uint32_t>(in_str[1]),
+                static_cast<uint32_t>(in_str[2]),
+                static_cast<uint32_t>(in_str[3]),
+                reserveSpace,
+                static_cast<uint32_t>(total_work),
+                static_cast<uint64_t>(in_offset),
+                static_cast<uint64_t>(out_offset),
+                static_cast<uint64_t>(rsvsp_offset));
+        }
+        else
+        {
+            handle.AddKernel(
+                kernel_name, network_config, program_name, kernel_name, vld, vgd, params)(
+                pstates,
+                dropout,
+                amp_scale,
+                static_cast<uint64_t>(in_len[1]),
+                static_cast<uint64_t>(in_len[2]),
+                static_cast<uint64_t>(in_len[3]),
+                static_cast<uint64_t>(in_len[4]),
+                y,
+                static_cast<uint64_t>(out_str[0]),
+                static_cast<uint64_t>(out_str[1]),
+                static_cast<uint64_t>(out_str[2]),
+                static_cast<uint64_t>(out_str[3]),
+                x,
+                static_cast<uint64_t>(in_str[0]),
+                static_cast<uint64_t>(in_str[1]),
+                static_cast<uint64_t>(in_str[2]),
+                static_cast<uint64_t>(in_str[3]),
+                reserveSpace,
+                static_cast<uint64_t>(total_work),
+                static_cast<uint64_t>(in_offset),
+                static_cast<uint64_t>(out_offset),
+                static_cast<uint64_t>(rsvsp_offset));
+        }
     }
 
     if(miopen::CheckNumericsEnabled())
@@ -368,211 +471,4 @@ void DropoutDescriptor::DropoutForward(const Handle& handle,
     }
 }
 
-void DropoutDescriptor::DropoutBackward(const Handle& handle,
-                                        const TensorDescriptor& noise_shape,
-                                        const TensorDescriptor& dyDesc,
-                                        ConstData_t dy,
-                                        const TensorDescriptor& dxDesc,
-                                        Data_t dx,
-                                        Data_t reserveSpace,
-                                        size_t reserveSpaceSizeInBytes,
-                                        size_t in_offset,
-                                        size_t out_offset,
-                                        size_t rsvsp_offset) const
-{
-    if(dx == nullptr || dy == nullptr)
-    {
-        MIOPEN_THROW(miopenStatusBadParm);
-    }
-
-    if(dxDesc.GetSize() != dyDesc.GetSize())
-    {
-        MIOPEN_THROW("Input/Output dimension does not match");
-    }
-
-    if(dyDesc.GetSize() > 5)
-    {
-        MIOPEN_THROW("Only support 1D to 5D tensors");
-    }
-
-    if(dxDesc.GetElementSize() != dyDesc.GetElementSize())
-    {
-        MIOPEN_THROW("Input/Output element size does not match");
-    }
-
-    if(dxDesc.GetElementSize() != noise_shape.GetElementSize() ||
-       dxDesc.GetSize() != noise_shape.GetSize())
-    {
-        MIOPEN_THROW("Only support dropout with regular noise shape currently");
-    }
-
-    if(dxDesc.GetType() != dyDesc.GetType())
-    {
-        MIOPEN_THROW("Input/Output datatype does not match");
-    }
-
-    if(dropout < 0.0 || dropout > 1.0)
-    {
-        MIOPEN_THROW("Invalid dropout rate");
-    }
-
-    bool use_prng = reserveSpace == nullptr;
-    if(((!use_prng || use_mask) &&
-        reserveSpaceSizeInBytes < dyDesc.GetElementSize() * sizeof(bool)) ||
-       (use_mask && use_prng))
-    {
-        MIOPEN_THROW("Insufficient reservespace size");
-    }
-
-    if(reserveSpaceSizeInBytes + dxDesc.GetElementSize() * GetTypeSize(dxDesc.GetType()) +
-           dyDesc.GetElementSize() * GetTypeSize(dyDesc.GetType()) >
-       handle.GetGlobalMemorySize())
-    {
-        MIOPEN_THROW("Memory required by dropout backward configs exceeds GPU memory range.");
-    }
-
-    if(miopen::CheckNumericsEnabled())
-    {
-        std::cout << "Dropout backward input numerics check at dropout rate " << dropout
-                  << std::endl;
-        miopen::checkNumericsInput(handle, dyDesc, dy);
-    }
-
-    // support up to 5D tensor
-    std::vector<size_t> in_len(5, 1);
-    std::vector<size_t> in_str(5, 1);
-    std::vector<size_t> out_len(5, 1);
-    std::vector<size_t> out_str(5, 1);
-
-    SquashPairedTensor(dxDesc.GetLengths(),
-                       dxDesc.GetStrides(),
-                       dyDesc.GetLengths(),
-                       dyDesc.GetStrides(),
-                       in_len,
-                       in_str,
-                       out_len,
-                       out_str);
-
-    size_t RD_BLCK    = /* (in_len[4] % 4 == 0) ? 4 : */ (in_len[2] % 2 == 0) ? 2 : 1;
-    size_t total_work = (in_len[4] / RD_BLCK) * in_len[3] * in_len[2] * in_len[1] * in_len[0];
-
-    size_t max_wk_grp = use_prng ? std::min(size_t(MAX_PRNG_STATE), handle.GetImage3dMaxWidth())
-                                 : size_t(MAX_WORKITEM_NUM);
-    size_t wk_grp_num =
-        std::min(max_wk_grp / 256,
-                 ((in_len[4] * in_len[3] * in_len[2] * in_len[1] * in_len[0] + 255) / 256));
-
-    if(use_prng)
-    {
-        size_t states_num = stateSizeInBytes / sizeof(prngStates);
-        if(states_num < wk_grp_num * 256)
-        {
-            MIOPEN_THROW("Insufficient state size for parallel PRNG");
-        }
-    }
-
-    std::string program_name = "MIOpenDropout.cl";
-    std::string kernel_name  = "DropoutBackward";
-
-    std::string network_config =
-        "bwd-" + std::string(dyDesc.GetType() == miopenHalf ? "fp16-" : "fp32-") + "-seed" +
-        std::to_string(seed) + "-rng" + std::to_string(rng_mode) + "-prng" +
-        std::to_string(static_cast<int>(use_prng)) + "-evo" +
-        std::to_string(static_cast<int>(state_evo)) + "-blk" + std::to_string(RD_BLCK) + "-wg" +
-        std::to_string(wk_grp_num) /* + "-noise" + std::to_string(noise_shape.GetLengths()[0]) */;
-
-    // TODO: Add noise shape
-    // for(int i = 1; i < noise_shape.GetSize(); i++)
-    //    network_config += "x" + std::to_string(noise_shape.GetLengths()[i]);
-
-    auto&& kernels = handle.GetKernels(kernel_name, network_config);
-
-    float amp_scale = float_equal(dropout, 1.0) ? 0 : 1 / (1 - dropout);
-    if(!kernels.empty())
-    {
-        kernels.front()(pstates,
-                        dropout,
-                        amp_scale,
-                        static_cast<int>(in_len[1]),
-                        static_cast<int>(in_len[2]),
-                        static_cast<int>(in_len[3]),
-                        static_cast<int>(in_len[4]),
-                        dy,
-                        static_cast<int>(out_str[0]),
-                        static_cast<int>(out_str[1]),
-                        static_cast<int>(out_str[2]),
-                        static_cast<int>(out_str[3]),
-                        dx,
-                        static_cast<int>(in_str[0]),
-                        static_cast<int>(in_str[1]),
-                        static_cast<int>(in_str[2]),
-                        static_cast<int>(in_str[3]),
-                        reserveSpace,
-                        static_cast<unsigned>(total_work),
-                        static_cast<unsigned>(in_offset),
-                        static_cast<unsigned>(out_offset),
-                        static_cast<unsigned>(rsvsp_offset));
-    }
-    else
-    {
-        std::string params;
-
-        const std::string data_type = GetDataType(dyDesc.GetType());
-        const std::string READ_DAT_TYPE =
-            RD_BLCK == 1 ? data_type : data_type + std::to_string(RD_BLCK);
-
-        params += " -DRD_BLCK=" + std::to_string(RD_BLCK) + " -DREAD_DAT_TYPE=" + READ_DAT_TYPE +
-                  " -DREAD_BOOL_TYPE=" +
-                  std::string(RD_BLCK == 4   ? "uint"
-                              : RD_BLCK == 2 ? "ushort"
-                                             : "uchar");
-
-        if(use_prng)
-        {
-            params += " -DUSE_PRNG=1";
-        }
-
-        if(dyDesc.GetType() == miopenHalf)
-            params += " -DMIOPEN_USE_FP16=1";
-        else
-            params += " -DMIOPEN_USE_FP32=1";
-
-        params += " -DRUN_FORWARD=0";
-
-        const std::vector<size_t> vld{256, 1, 1};
-        const std::vector<size_t> vgd{wk_grp_num * 256, 1, 1};
-
-        handle.AddKernel(kernel_name, network_config, program_name, kernel_name, vld, vgd, params)(
-            pstates,
-            dropout,
-            amp_scale,
-            static_cast<int>(in_len[1]),
-            static_cast<int>(in_len[2]),
-            static_cast<int>(in_len[3]),
-            static_cast<int>(in_len[4]),
-            dy,
-            static_cast<int>(out_str[0]),
-            static_cast<int>(out_str[1]),
-            static_cast<int>(out_str[2]),
-            static_cast<int>(out_str[3]),
-            dx,
-            static_cast<int>(in_str[0]),
-            static_cast<int>(in_str[1]),
-            static_cast<int>(in_str[2]),
-            static_cast<int>(in_str[3]),
-            reserveSpace,
-            static_cast<unsigned>(total_work),
-            static_cast<unsigned>(in_offset),
-            static_cast<unsigned>(out_offset),
-            static_cast<unsigned>(rsvsp_offset));
-    }
-
-    if(miopen::CheckNumericsEnabled())
-    {
-        std::cout << "Dropout backward output numerics check at dropout rate " << dropout
-                  << std::endl;
-        miopen::checkNumericsOutput(handle, dxDesc, dx);
-    }
-}
-
 } // namespace miopen
