@@ -37,6 +37,7 @@
 
 #include <cassert>
 #include <cstddef>
+#include <memory>
 #include <algorithm>
 #include <cmath>
 #include <ostream>
@@ -235,6 +236,7 @@ inline int GetDataTypeSize(miopenDataType_t t)
     case miopenInt8: return (1);
     case miopenBFloat16: return (2);
     case miopenInt32: return (4);
+    case miopenInt64:
     default: MIOPEN_THROW("Only float, half, double, bfloat16, int8 data types are supported.");
     };
 };
@@ -292,6 +294,7 @@ inline int GetDataTypeId(miopenDataType_t t)
     case miopenFloat8:
     case miopenBFloat8:
     case miopenInt32: return (static_cast<int>('O'));
+    case miopenInt64:
     default: MIOPEN_THROW("Only float, half, bfloat16, float8, bfloat8 data type is supported.");
     };
 };
@@ -331,6 +334,7 @@ static ck::DataTypeEnum_t mapDataTypeId(miopenDataType_t t)
     case miopenInt32: return DataTypeEnum_t::Int32;
     case miopenFloat8:
     case miopenBFloat8:
+    case miopenInt64:
     default: MIOPEN_THROW("Only float, half, double data type is supported.");
     };
 };
@@ -520,6 +524,10 @@ ReduceTensorDescriptor::ReduceTensorDescriptor(miopenReduceTensorOp_t reduceTens
         MIOPEN_THROW("Only int32 type is supported for ReduceTensor indices.");
 };
 
+// This is WS requirement of the dynamic reduction.
+// We must enforce it especially when reduction is used internally.
+constexpr std::size_t workspaceAlignRequirementBytes = 64;
+
 // return the size of the workspace in bytes, so that the workspace buffer can be prepared by the
 // user
 std::size_t ReduceTensorDescriptor::GetWorkspaceSize(const Handle& handle,
@@ -546,7 +554,7 @@ std::size_t ReduceTensorDescriptor::GetWorkspaceSize(const Handle& handle,
 
     int blockSize;
 
-    if(!miopen::IsDisabled(ENV(MIOPEN_DEBUG_DYNAMIC_REDUCTION)))
+    if(!env::disabled(MIOPEN_DEBUG_DYNAMIC_REDUCTION))
     {
         const tunable_generic_reduction* tunable = &default_tunable_generic_reduction;
         blockSize                                = tunable->BlockSize;
@@ -568,10 +576,10 @@ std::size_t ReduceTensorDescriptor::GetWorkspaceSize(const Handle& handle,
     std::size_t wsSizeInBytes =
         !need_indices ? workspace_size * detail::GetDataTypeSize(inDesc.GetType())
                       : workspace_size * (detail::GetDataTypeSize(inDesc.GetType()) + sizeof(int)) +
-                            64 + sizeof(int);
+                            64 + sizeof(int) + workspaceAlignRequirementBytes;
 
     // dynamic reduction use one additional page for storing tensor descriptors
-    if(!miopen::IsDisabled(ENV(MIOPEN_DEBUG_DYNAMIC_REDUCTION)))
+    if(!env::disabled(MIOPEN_DEBUG_DYNAMIC_REDUCTION))
         wsSizeInBytes += 4096;
 
     return (wsSizeInBytes);
@@ -637,8 +645,7 @@ void ReduceTensorDescriptor::ReduceTensor(const Handle& handle,
 
     const tunable_generic_reduction* tunable = &default_tunable_generic_reduction;
 
-    const int blockSize =
-        !miopen::IsDisabled(ENV(MIOPEN_DEBUG_DYNAMIC_REDUCTION)) ? tunable->BlockSize : 256;
+    const int blockSize = !env::disabled(MIOPEN_DEBUG_DYNAMIC_REDUCTION) ? tunable->BlockSize : 256;
     detail::ReductionKernelConfigurator configurator(blockSize, handle.GetWavefrontWidth());
 
     const bool need_indices =
@@ -721,7 +728,7 @@ void ReduceTensorDescriptor::ReduceTensor(const Handle& handle,
                          ? static_cast<float>(*reinterpret_cast<const double*>(beta))
                          : *reinterpret_cast<const float*>(beta);
 
-    if(miopen::IsDisabled(ENV(MIOPEN_DEBUG_DYNAMIC_REDUCTION)))
+    if(env::disabled(MIOPEN_DEBUG_DYNAMIC_REDUCTION))
     { // use static reduction
         std::vector<std::size_t> invariantLengths;
         std::vector<std::size_t> invariantStrides;
@@ -1031,6 +1038,14 @@ void ReduceTensorDescriptor::ReduceTensor(const Handle& handle,
                                        std::to_string(static_cast<int>(use_padding.first)) +
                                        std::to_string(static_cast<int>(use_padding.second));
 
+        if(nullptr == std::align(workspaceAlignRequirementBytes,
+                                 ws_sizeInBytes - workspaceAlignRequirementBytes,
+                                 workspace,
+                                 workspaceSizeInBytes))
+        {
+            MIOPEN_THROW(miopenStatusInternalError, "Alignment failed. There is not enough space.");
+        }
+
         if(!reduceAllDims)
         {
             handle.AddKernel(
