@@ -281,6 +281,9 @@ struct verify_forward_train_3d_bn_per_activation
                                          BuildReshaped4DTensorDescriptor(out.desc),
                                          out_dev.get(),
                                          BuildReshaped4DTensorDescriptor(scale.desc),
+                                         BuildReshaped4DTensorDescriptor(shift.desc),
+                                         BuildReshaped4DTensorDescriptor(shift.desc),
+                                         BuildReshaped4DTensorDescriptor(shift.desc),
                                          scale_dev.get(),
                                          shift_dev.get(),
                                          expAvgFactor,
@@ -450,6 +453,9 @@ struct verify_forward_infer_3d_bn_per_activation_recalc
                                           BuildReshaped4DTensorDescriptor(out.desc),
                                           out_dev.get(),
                                           BuildReshaped4DTensorDescriptor(scale.desc),
+                                          BuildReshaped4DTensorDescriptor(shift.desc),
+                                          BuildReshaped4DTensorDescriptor(shift.desc),
+                                          BuildReshaped4DTensorDescriptor(shift.desc),
                                           scale_dev.get(),
                                           shift_dev.get(),
                                           nullptr,
@@ -573,6 +579,9 @@ struct verify_forward_infer_3d_bn_per_activation_use_est
                                           BuildReshaped4DTensorDescriptor(out.desc),
                                           out_dev.get(),
                                           BuildReshaped4DTensorDescriptor(scale.desc),
+                                          BuildReshaped4DTensorDescriptor(shift.desc),
+                                          BuildReshaped4DTensorDescriptor(shift.desc),
+                                          BuildReshaped4DTensorDescriptor(shift.desc),
                                           scale_dev.get(),
                                           shift_dev.get(),
                                           estMean_dev.get(),
@@ -747,6 +756,9 @@ struct verify_backward_3d_bn_per_activation_use_saved
                                   BuildReshaped4DTensorDescriptor(dx_out.desc),
                                   dx_out_dev.get(),
                                   BuildReshaped4DTensorDescriptor(scale.desc),
+                                  BuildReshaped4DTensorDescriptor(dshift.desc),
+                                  BuildReshaped4DTensorDescriptor(dshift.desc),
+                                  BuildReshaped4DTensorDescriptor(dshift.desc),
                                   scale_dev.get(),
                                   dscale_dev.get(),
                                   dshift_dev.get(),
@@ -948,6 +960,9 @@ struct verify_backward_3d_bn_per_activation_recalc
                                   BuildReshaped4DTensorDescriptor(dx_out.desc),
                                   dx_out_dev.get(),
                                   BuildReshaped4DTensorDescriptor(scale.desc),
+                                  BuildReshaped4DTensorDescriptor(dshift.desc),
+                                  BuildReshaped4DTensorDescriptor(dshift.desc),
+                                  BuildReshaped4DTensorDescriptor(dshift.desc),
                                   scale_dev.get(),
                                   dscale_dev.get(),
                                   dshift_dev.get(),
