@@ -36,37 +36,13 @@
 #include <cstdlib>
 #include <ctime>
 #include <type_traits>
-#if !defined(_WIN32)
 #include <half/half.hpp>
-#else
-#include <half.hpp>
-#endif
 #include "test.hpp"
 #include "driver.hpp"
 #include "tensor_holder.hpp"
 #include "cpu_conv.hpp"
 #include "random.hpp"
 
-enum tensor_layout_t
-{
-    miopen_tensor_layout_nchw,
-    miopen_tensor_layout_ncdhw,
-    miopen_tensor_layout_nhwc,
-    miopen_tensor_layout_ndhwc,
-};
-
-std::string tensor_layout_to_string(tensor_layout_t layout)
-{
-    switch(layout)
-    {
-    case miopen_tensor_layout_nchw: return "NCHW";
-    case miopen_tensor_layout_ncdhw: return "NCDHW";
-    case miopen_tensor_layout_nhwc: return "NHWC";
-    case miopen_tensor_layout_ndhwc: return "NDHWC";
-    default: MIOPEN_THROW("Unsupported tensor layout");
-    }
-}
-
 struct gpu_reference_kernel_base
 {
     miopenHandle_t handle{};
@@ -315,7 +291,7 @@ void pad_tensor_strides(std::vector<int>& strides)
 template <miopen::conv::Direction direction,
           typename TRef,
           typename Tout,
-          tensor_layout_t tensor_layout>
+          miopenTensorLayout_t tensor_layout>
 struct gpu_reference_conv_2d : gpu_reference_kernel_base
 {
     void run()
@@ -353,7 +329,7 @@ struct gpu_reference_conv_2d : gpu_reference_kernel_base
             std::vector<int> out_strides;
 
             std::string layout_default = miopen::tensor_layout_get_default(4);
-            std::string layout_string  = tensor_layout_to_string(tensor_layout);
+            std::string layout_string  = miopen::TensorDescriptor::LayoutEnumToStr(tensor_layout);
 
             miopen::tensor_layout_to_strides(in_len, layout_default, layout_string, in_strides);
             miopen::tensor_layout_to_strides(wei_len, layout_default, layout_string, wei_strides);
@@ -677,7 +653,7 @@ struct gpu_reference_conv_2d : gpu_reference_kernel_base
 template <miopen::conv::Direction direction,
           typename TRef,
           typename Tout,
-          tensor_layout_t tensor_layout>
+          miopenTensorLayout_t tensor_layout>
 struct gpu_reference_conv_3d : gpu_reference_kernel_base
 {
     void run()
@@ -721,7 +697,7 @@ struct gpu_reference_conv_3d : gpu_reference_kernel_base
             std::vector<int> out_strides;
 
             std::string layout_default = miopen::tensor_layout_get_default(5);
-            std::string layout_string  = tensor_layout_to_string(tensor_layout);
+            std::string layout_string  = miopen::TensorDescriptor::LayoutEnumToStr(tensor_layout);
 
             miopen::tensor_layout_to_strides(in_len, layout_default, layout_string, in_strides);
             miopen::tensor_layout_to_strides(wei_len, layout_default, layout_string, wei_strides);
@@ -1043,186 +1019,174 @@ struct gpu_reference_conv_3d : gpu_reference_kernel_base
 int main()
 {
     // 2d NCHW
-    run_test<gpu_reference_conv_2d<miopen::conv::Direction::Forward,
-                                   float,
-                                   float,
-                                   miopen_tensor_layout_nchw>>();
+    run_test<
+        gpu_reference_conv_2d<miopen::conv::Direction::Forward, float, float, miopenTensorNCHW>>();
     run_test<gpu_reference_conv_2d<miopen::conv::Direction::Forward,
                                    half_float::half,
                                    half_float::half,
-                                   miopen_tensor_layout_nchw>>();
+                                   miopenTensorNCHW>>();
     run_test<gpu_reference_conv_2d<miopen::conv::Direction::Forward,
                                    bfloat16,
                                    bfloat16,
-                                   miopen_tensor_layout_nchw>>();
+                                   miopenTensorNCHW>>();
     run_test<gpu_reference_conv_2d<miopen::conv::Direction::Forward,
                                    int8_t,
                                    int32_t,
-                                   miopen_tensor_layout_nchw>>();
-    run_test<gpu_reference_conv_2d<miopen::conv::Direction::Forward,
-                                   int8_t,
-                                   float,
-                                   miopen_tensor_layout_nchw>>();
+                                   miopenTensorNCHW>>();
+    run_test<
+        gpu_reference_conv_2d<miopen::conv::Direction::Forward, int8_t, float, miopenTensorNCHW>>();
     run_test<gpu_reference_conv_2d<miopen::conv::Direction::BackwardData,
                                    float,
                                    float,
-                                   miopen_tensor_layout_nchw>>();
+                                   miopenTensorNCHW>>();
     run_test<gpu_reference_conv_2d<miopen::conv::Direction::BackwardData,
                                    half_float::half,
                                    half_float::half,
-                                   miopen_tensor_layout_nchw>>();
+                                   miopenTensorNCHW>>();
     run_test<gpu_reference_conv_2d<miopen::conv::Direction::BackwardData,
                                    bfloat16,
                                    bfloat16,
-                                   miopen_tensor_layout_nchw>>();
+                                   miopenTensorNCHW>>();
     run_test<gpu_reference_conv_2d<miopen::conv::Direction::BackwardWeights,
                                    float,
                                    float,
-                                   miopen_tensor_layout_nchw>>();
+                                   miopenTensorNCHW>>();
     run_test<gpu_reference_conv_2d<miopen::conv::Direction::BackwardWeights,
                                    half_float::half,
                                    half_float::half,
-                                   miopen_tensor_layout_nchw>>();
+                                   miopenTensorNCHW>>();
     run_test<gpu_reference_conv_2d<miopen::conv::Direction::BackwardWeights,
                                    bfloat16,
                                    bfloat16,
-                                   miopen_tensor_layout_nchw>>();
+                                   miopenTensorNCHW>>();
 
     // 3d NCDHW
-    run_test<gpu_reference_conv_3d<miopen::conv::Direction::Forward,
-                                   float,
-                                   float,
-                                   miopen_tensor_layout_ncdhw>>();
+    run_test<
+        gpu_reference_conv_3d<miopen::conv::Direction::Forward, float, float, miopenTensorNCDHW>>();
     run_test<gpu_reference_conv_3d<miopen::conv::Direction::Forward,
                                    half_float::half,
                                    half_float::half,
-                                   miopen_tensor_layout_ncdhw>>();
+                                   miopenTensorNCDHW>>();
     run_test<gpu_reference_conv_3d<miopen::conv::Direction::Forward,
                                    bfloat16,
                                    bfloat16,
-                                   miopen_tensor_layout_ncdhw>>();
+                                   miopenTensorNCDHW>>();
     run_test<gpu_reference_conv_3d<miopen::conv::Direction::Forward,
                                    int8_t,
                                    int32_t,
-                                   miopen_tensor_layout_ncdhw>>();
+                                   miopenTensorNCDHW>>();
     run_test<gpu_reference_conv_3d<miopen::conv::Direction::Forward,
                                    int8_t,
                                    float,
-                                   miopen_tensor_layout_ncdhw>>();
+                                   miopenTensorNCDHW>>();
     run_test<gpu_reference_conv_3d<miopen::conv::Direction::BackwardData,
                                    float,
                                    float,
-                                   miopen_tensor_layout_ncdhw>>();
+                                   miopenTensorNCDHW>>();
     run_test<gpu_reference_conv_3d<miopen::conv::Direction::BackwardData,
                                    half_float::half,
                                    half_float::half,
-                                   miopen_tensor_layout_ncdhw>>();
+                                   miopenTensorNCDHW>>();
     run_test<gpu_reference_conv_3d<miopen::conv::Direction::BackwardData,
                                    bfloat16,
                                    bfloat16,
-                                   miopen_tensor_layout_ncdhw>>();
+                                   miopenTensorNCDHW>>();
     run_test<gpu_reference_conv_3d<miopen::conv::Direction::BackwardWeights,
                                    float,
                                    float,
-                                   miopen_tensor_layout_ncdhw>>();
+                                   miopenTensorNCDHW>>();
     run_test<gpu_reference_conv_3d<miopen::conv::Direction::BackwardWeights,
                                    half_float::half,
                                    half_float::half,
-                                   miopen_tensor_layout_ncdhw>>();
+                                   miopenTensorNCDHW>>();
     run_test<gpu_reference_conv_3d<miopen::conv::Direction::BackwardWeights,
                                    bfloat16,
                                    bfloat16,
-                                   miopen_tensor_layout_ncdhw>>();
+                                   miopenTensorNCDHW>>();
 
     // 2d NHWC
-    run_test<gpu_reference_conv_2d<miopen::conv::Direction::Forward,
-                                   float,
-                                   float,
-                                   miopen_tensor_layout_nhwc>>();
+    run_test<
+        gpu_reference_conv_2d<miopen::conv::Direction::Forward, float, float, miopenTensorNHWC>>();
     run_test<gpu_reference_conv_2d<miopen::conv::Direction::Forward,
                                    half_float::half,
                                    half_float::half,
-                                   miopen_tensor_layout_nhwc>>();
+                                   miopenTensorNHWC>>();
     run_test<gpu_reference_conv_2d<miopen::conv::Direction::Forward,
                                    bfloat16,
                                    bfloat16,
-                                   miopen_tensor_layout_nhwc>>();
+                                   miopenTensorNHWC>>();
     run_test<gpu_reference_conv_2d<miopen::conv::Direction::Forward,
                                    int8_t,
                                    int32_t,
-                                   miopen_tensor_layout_nhwc>>();
-    run_test<gpu_reference_conv_2d<miopen::conv::Direction::Forward,
-                                   int8_t,
-                                   float,
-                                   miopen_tensor_layout_nhwc>>();
+                                   miopenTensorNHWC>>();
+    run_test<
+        gpu_reference_conv_2d<miopen::conv::Direction::Forward, int8_t, float, miopenTensorNHWC>>();
     run_test<gpu_reference_conv_2d<miopen::conv::Direction::BackwardData,
                                    float,
                                    float,
-                                   miopen_tensor_layout_nhwc>>();
+                                   miopenTensorNHWC>>();
     run_test<gpu_reference_conv_2d<miopen::conv::Direction::BackwardData,
                                    half_float::half,
                                    half_float::half,
-                                   miopen_tensor_layout_nhwc>>();
+                                   miopenTensorNHWC>>();
     run_test<gpu_reference_conv_2d<miopen::conv::Direction::BackwardData,
                                    bfloat16,
                                    bfloat16,
-                                   miopen_tensor_layout_nhwc>>();
+                                   miopenTensorNHWC>>();
     run_test<gpu_reference_conv_2d<miopen::conv::Direction::BackwardWeights,
                                    float,
                                    float,
-                                   miopen_tensor_layout_nhwc>>();
+                                   miopenTensorNHWC>>();
     run_test<gpu_reference_conv_2d<miopen::conv::Direction::BackwardWeights,
                                    half_float::half,
                                    half_float::half,
-                                   miopen_tensor_layout_nhwc>>();
+                                   miopenTensorNHWC>>();
     run_test<gpu_reference_conv_2d<miopen::conv::Direction::BackwardWeights,
                                    bfloat16,
                                    bfloat16,
-                                   miopen_tensor_layout_nhwc>>();
+                                   miopenTensorNHWC>>();
 
     // 3d NDHWC
-    run_test<gpu_reference_conv_3d<miopen::conv::Direction::Forward,
-                                   float,
-                                   float,
-                                   miopen_tensor_layout_ndhwc>>();
+    run_test<
+        gpu_reference_conv_3d<miopen::conv::Direction::Forward, float, float, miopenTensorNDHWC>>();
     run_test<gpu_reference_conv_3d<miopen::conv::Direction::Forward,
                                    half_float::half,
                                    half_float::half,
-                                   miopen_tensor_layout_ndhwc>>();
+                                   miopenTensorNDHWC>>();
     run_test<gpu_reference_conv_3d<miopen::conv::Direction::Forward,
                                    bfloat16,
                                    bfloat16,
-                                   miopen_tensor_layout_ndhwc>>();
+                                   miopenTensorNDHWC>>();
     run_test<gpu_reference_conv_3d<miopen::conv::Direction::Forward,
                                    int8_t,
                                    int32_t,
-                                   miopen_tensor_layout_ndhwc>>();
+                                   miopenTensorNDHWC>>();
     run_test<gpu_reference_conv_3d<miopen::conv::Direction::Forward,
                                    int8_t,
                                    float,
-                                   miopen_tensor_layout_ndhwc>>();
+                                   miopenTensorNDHWC>>();
     run_test<gpu_reference_conv_3d<miopen::conv::Direction::BackwardData,
                                    float,
                                    float,
-                                   miopen_tensor_layout_ndhwc>>();
+                                   miopenTensorNDHWC>>();
     run_test<gpu_reference_conv_3d<miopen::conv::Direction::BackwardData,
                                    half_float::half,
                                    half_float::half,
-                                   miopen_tensor_layout_ndhwc>>();
+                                   miopenTensorNDHWC>>();
     run_test<gpu_reference_conv_3d<miopen::conv::Direction::BackwardData,
                                    bfloat16,
                                    bfloat16,
-                                   miopen_tensor_layout_ndhwc>>();
+                                   miopenTensorNDHWC>>();
     run_test<gpu_reference_conv_3d<miopen::conv::Direction::BackwardWeights,
                                    float,
                                    float,
-                                   miopen_tensor_layout_ndhwc>>();
+                                   miopenTensorNDHWC>>();
     run_test<gpu_reference_conv_3d<miopen::conv::Direction::BackwardWeights,
                                    half_float::half,
                                    half_float::half,
-                                   miopen_tensor_layout_ndhwc>>();
+                                   miopenTensorNDHWC>>();
     run_test<gpu_reference_conv_3d<miopen::conv::Direction::BackwardWeights,
                                    bfloat16,
                                    bfloat16,
-                                   miopen_tensor_layout_ndhwc>>();
+                                   miopenTensorNDHWC>>();
 }
